! Data Assimilation Research Testbed -- DART
! Copyright 2004, 2005, Data Assimilation Initiative, University Corporation for Atmospheric Research
! Licensed under the GPL -- www.gpl.org/licenses/gpl.html

module model_mod

! <next six lines automatically updated by CVS, do not edit>
! $Source: /home/thoar/CVS.REPOS/DART/models/wrf/model_mod.f90,v $
! $Revision: 1.48 $
! $Date: 2005/02/26 06:14:25 $
! $Author: thoar $
! $Id: model_mod.f90,v 1.48 2005/02/26 06:14:25 thoar Exp $
! $Name: hawaii $

! Assimilation interface for WRF model

!-----------------------------------------------------------------------
!
!     interface for WRF
!
!-----------------------------------------------------------------------
!---------------- m o d u l e   i n f o r m a t i o n ------------------
!-----------------------------------------------------------------------

use         types_mod, only : r8, deg2rad, missing_r8, ps0, earth_radius, &
                              gas_constant, gas_constant_v
use  time_manager_mod, only : time_type, set_time, set_calendar_type, GREGORIAN
use      location_mod, only : location_type, get_location, set_location, &
                              get_dist, horiz_dist_only, &
                              LocationDims, LocationName, LocationLName, &
                              query_location
use     utilities_mod, only : file_exist, open_file, check_nml_error, &
                              close_file, &
                              register_module, error_handler, E_ERR, &
                              E_MSG, logfileunit
use      obs_kind_mod, only : KIND_U, KIND_V, KIND_PS, KIND_T, KIND_QV, &
                              KIND_P, KIND_W, KIND_QR, KIND_TD, KIND_RHO, &
                              KIND_VR, KIND_REF, KIND_U10, KIND_V10, KIND_T2, &
                              KIND_Q2, KIND_TD2
use         map_utils, only : proj_info, map_init, map_set, latlon_to_ij, &
                              PROJ_LATLON, PROJ_MERC, PROJ_LC, PROJ_PS, &
                              gridwind_to_truewind
use netcdf
use typesizes

implicit none
private

!  public routines and data for the WRF model

public     get_model_size,                    &
           get_state_meta_data,               &
           model_interpolate,                 &
           get_model_time_step,               &
           static_init_model,                 &
           model_get_close_states,            &
           pert_model_state,                  &
           nc_write_model_atts,               &
           nc_write_model_vars

!  public stubs 

public     adv_1step,       &
           end_model,       &
           init_time,       &
           init_conditions

!-----------------------------------------------------------------------
! CVS Generated file description for error handling, do not edit
character(len=128) :: &
source   = "$Source: /home/thoar/CVS.REPOS/DART/models/wrf/model_mod.f90,v $", &
revision = "$Revision: 1.48 $", &
revdate  = "$Date: 2005/02/26 06:14:25 $"

!-----------------------------------------------------------------------
! Model namelist parameters with default values.
!-----------------------------------------------------------------------

logical :: output_state_vector  = .true.     ! output prognostic variables
integer :: num_moist_vars       = 0
integer :: num_domains          = 1
integer :: calendar_type        = GREGORIAN
logical :: surf_obs             = .false.
character(len = 72) :: adv_mod_command = 'wrf.exe'

namelist /model_nml/ output_state_vector, num_moist_vars, &
                     num_domains, calendar_type, surf_obs, &
                     adv_mod_command

!-----------------------------------------------------------------------

! Private definition of model variable types

integer, parameter :: TYPE_U   = 1,   TYPE_V   = 2,  TYPE_W  = 3,  &
                      TYPE_GZ  = 4,   TYPE_T   = 5,  TYPE_MU = 6,  &
                      TYPE_QV  = 7,   TYPE_QC  = 8,  TYPE_QR = 9,  &
                      TYPE_QI  = 10,  TYPE_QS  = 11, TYPE_QG = 12, &
                      TYPE_U10 = 13,  TYPE_V10 = 14, TYPE_T2 = 15, &
                      TYPE_Q2  = 16,  TYPE_PS  = 17, TYPE_TSLB = 18, &
                      TYPE_TSK = 19


real (kind=r8), PARAMETER    :: cp = 7.0_r8*gas_constant/2.0_r8
real (kind=r8), PARAMETER    :: rd_over_rv = gas_constant / gas_constant_v
real (kind=r8), PARAMETER    :: kappa = gas_constant / cp
real (kind=r8), PARAMETER    :: ts0 = 300.0_r8        ! Base potential temperature for all levels.

real (kind=r8), PARAMETER    :: gravity = 9.81_r8

!---- private data ----

TYPE wrf_static_data_for_dart

   integer  :: bt, bts, sn, sns, we, wes, sls
   real(r8) :: dx, dy, dt
   integer  :: map_proj
   real(r8) :: cen_lat,cen_lon
   type(proj_info) :: proj

   integer  :: n_moist
   logical  :: surf_obs
   real(r8), dimension(:),     pointer :: znu, dn, dnw, zs
   real(r8), dimension(:,:),   pointer :: mub, latitude, longitude, hgt
   real(r8), dimension(:,:),   pointer :: mapfac_m, mapfac_u, mapfac_v
   real(r8), dimension(:,:,:), pointer :: phb

   integer :: number_of_wrf_variables
   integer, dimension(:,:), pointer :: var_index
   integer, dimension(:,:), pointer :: var_size
   integer, dimension(:),   pointer :: var_type
   integer, dimension(:,:), pointer :: land

end type wrf_static_data_for_dart

type wrf_dom
   type(wrf_static_data_for_dart), pointer :: dom(:)
   integer :: model_size
end type wrf_dom

type(wrf_dom) :: wrf


contains

!#######################################################################

subroutine static_init_model()

! INitializes class data for WRF???

integer :: ncid
integer :: io, ierr, iunit

character (len=80)    :: name
character (len=1)     :: idom
logical, parameter    :: debug = .false.
integer               :: var_id, ind, i, id, dart_index

integer  :: proj_code
real(r8) :: stdlon,truelat1,truelat2

!----------------------------------------------------------------------

! Register the module
call register_module(source, revision, revdate)

! Reading the namelist input
if(file_exist('input.nml')) then

   iunit = open_file('input.nml', action = 'read')
   read(iunit, nml = model_nml, iostat = io )
   ierr = check_nml_error(io, 'model_nml')
   call close_file(iunit)

endif

! Record the namelist values used for the run ...
call error_handler(E_MSG,'static_init_model','model_nml values are',' ',' ',' ')
write(logfileunit, nml=model_nml)
write(     *     , nml=model_nml)

allocate(wrf%dom(num_domains))

wrf%dom(:)%n_moist = num_moist_vars

if( num_moist_vars > 6) then
   write(*,'(''num_moist_vars = '',i3)')num_moist_vars
   call error_handler(E_ERR,'static_init_model', &
        'num_moist_vars is too large.', source, revision,revdate)
endif

wrf%dom(:)%surf_obs = surf_obs

if ( debug ) then
   if ( output_state_vector ) then
      write(*,*)'netcdf file in state vector format'
   else
      write(*,*)'netcdf file in prognostic vector format'
   endif
endif

call set_calendar_type(calendar_type)

dart_index = 0

do id=1,num_domains

   write( idom , '(I1)') id

   call check( nf90_open('wrfinput_d0'//idom, NF90_NOWRITE, ncid) )
   if(debug) write(6,*) ' ncid is ',ncid

! get wrf grid dimensions

   call check( nf90_inq_dimid(ncid, "bottom_top", var_id) )
   call check( nf90_inquire_dimension(ncid, var_id, name, wrf%dom(id)%bt) )

   call check( nf90_inq_dimid(ncid, "bottom_top_stag", var_id) ) ! reuse var_id, no harm
   call check( nf90_inquire_dimension(ncid, var_id, name, wrf%dom(id)%bts) )

   call check( nf90_inq_dimid(ncid, "south_north", var_id) )
   call check( nf90_inquire_dimension(ncid, var_id, name, wrf%dom(id)%sn) )

   call check( nf90_inq_dimid(ncid, "south_north_stag", var_id)) ! reuse var_id, no harm
   call check( nf90_inquire_dimension(ncid, var_id, name, wrf%dom(id)%sns) )

   call check( nf90_inq_dimid(ncid, "west_east", var_id) )
   call check( nf90_inquire_dimension(ncid, var_id, name, wrf%dom(id)%we) )

   call check( nf90_inq_dimid(ncid, "west_east_stag", var_id) )  ! reuse var_id, no harm
   call check( nf90_inquire_dimension(ncid, var_id, name, wrf%dom(id)%wes) )

   call check( nf90_inq_dimid(ncid, "soil_layers_stag", var_id) )  ! reuse var_id, no harm
   call check( nf90_inquire_dimension(ncid, var_id, name, wrf%dom(id)%sls) )

   if(debug) then
      write(6,*) ' dimensions bt, sn, we are ',wrf%dom(id)%bt, &
           wrf%dom(id)%sn, wrf%dom(id)%we
      write(6,*) ' staggered  bt, sn, we are ',wrf%dom(id)%bts, &
           wrf%dom(id)%sns,wrf%dom(id)%wes
   endif

! get meta data and static data we need

   call check( nf90_get_att(ncid, nf90_global, 'DX', wrf%dom(id)%dx) )
   call check( nf90_get_att(ncid, nf90_global, 'DY', wrf%dom(id)%dy) )
   call check( nf90_get_att(ncid, nf90_global, 'DT', wrf%dom(id)%dt) )
   print*,'dt from wrfinput is: ',wrf%dom(id)%dt
   call read_dt_from_wrf_nml(wrf%dom(id)%dt)
   print*,'Using dt from namelist.input: ',wrf%dom(id)%dt
   if(debug) write(6,*) ' dx, dy, dt are ',wrf%dom(id)%dx, &
        wrf%dom(id)%dy, wrf%dom(id)%dt

   call check( nf90_get_att(ncid, nf90_global, 'MAP_PROJ', wrf%dom(id)%map_proj) )
   if(debug) write(6,*) ' map_proj is ',wrf%dom(id)%map_proj

   call check( nf90_get_att(ncid, nf90_global, 'CEN_LAT', wrf%dom(id)%cen_lat) )
   if(debug) write(6,*) ' cen_lat is ',wrf%dom(id)%cen_lat

   call check( nf90_get_att(ncid, nf90_global, 'CEN_LON', wrf%dom(id)%cen_lon) )

   call check( nf90_get_att(ncid, nf90_global, 'TRUELAT1', truelat1) )
   if(debug) write(6,*) ' truelat1 is ',truelat1

   call check( nf90_get_att(ncid, nf90_global, 'TRUELAT2', truelat2) )
   if(debug) write(6,*) ' truelat2 is ',truelat2

   call check( nf90_get_att(ncid, nf90_global, 'STAND_LON', stdlon) )

!  get 1D (z) static data defining grid levels

   allocate(wrf%dom(id)%dn(1:wrf%dom(id)%bt))
   call check( nf90_inq_varid(ncid, "DN", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%dn) )
   if(debug) write(6,*) ' dn ',wrf%dom(id)%dn

   allocate(wrf%dom(id)%znu(1:wrf%dom(id)%bt))
   call check( nf90_inq_varid(ncid, "ZNU", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%znu) )
   if(debug) write(6,*) ' znu is ',wrf%dom(id)%znu

   allocate(wrf%dom(id)%dnw(1:wrf%dom(id)%bt))
   call check( nf90_inq_varid(ncid, "DNW", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%dnw) )
   if(debug) write(6,*) ' dnw is ',wrf%dom(id)%dnw

   allocate(wrf%dom(id)%zs(1:wrf%dom(id)%sls))
   call check( nf90_inq_varid(ncid, "ZS", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%zs) )

!  get 2D (x,y) base state for mu, latitude, longitude

   allocate(wrf%dom(id)%mub(1:wrf%dom(id)%we,1:wrf%dom(id)%sn))
   call check( nf90_inq_varid(ncid, "MUB", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%mub) )
   if(debug) then
      write(6,*) ' corners of mub '
      write(6,*) wrf%dom(id)%mub(1,1),wrf%dom(id)%mub(wrf%dom(id)%we,1),  &
           wrf%dom(id)%mub(1,wrf%dom(id)%sn),wrf%dom(id)%mub(wrf%dom(id)%we, &
           wrf%dom(id)%sn)
   end if

   allocate(wrf%dom(id)%longitude(1:wrf%dom(id)%we,1:wrf%dom(id)%sn))
   call check( nf90_inq_varid(ncid, "XLONG", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%longitude) )

   allocate(wrf%dom(id)%latitude(1:wrf%dom(id)%we,1:wrf%dom(id)%sn))
   call check( nf90_inq_varid(ncid, "XLAT", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%latitude) )

   allocate(wrf%dom(id)%land(1:wrf%dom(id)%we,1:wrf%dom(id)%sn))
   call check( nf90_inq_varid(ncid, "XLAND", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%land) )
   if(debug) then
      write(6,*) ' corners of land '
      write(6,*) wrf%dom(id)%land(1,1),wrf%dom(id)%land(wrf%dom(id)%we,1),  &
           wrf%dom(id)%land(1,wrf%dom(id)%sn),wrf%dom(id)%land(wrf%dom(id)%we, &
           wrf%dom(id)%sn)
   end if

   if(debug) then
      write(6,*) ' corners of lat '
      write(6,*) wrf%dom(id)%latitude(1,1),wrf%dom(id)%latitude(wrf%dom(id)%we,1),  &
           wrf%dom(id)%latitude(1,wrf%dom(id)%sn), &
           wrf%dom(id)%latitude(wrf%dom(id)%we,wrf%dom(id)%sn)
      write(6,*) ' corners of long '
      write(6,*) wrf%dom(id)%longitude(1,1),wrf%dom(id)%longitude(wrf%dom(id)%we,1),  &
           wrf%dom(id)%longitude(1,wrf%dom(id)%sn), &
           wrf%dom(id)%longitude(wrf%dom(id)%we,wrf%dom(id)%sn)
   end if

   allocate(wrf%dom(id)%mapfac_m(1:wrf%dom(id)%we,1:wrf%dom(id)%sn))
   call check( nf90_inq_varid(ncid, "MAPFAC_M", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%mapfac_m) )

   allocate(wrf%dom(id)%hgt(1:wrf%dom(id)%we,1:wrf%dom(id)%sn))
   call check( nf90_inq_varid(ncid, "HGT", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%hgt) )

   allocate(wrf%dom(id)%mapfac_u(1:wrf%dom(id)%wes,1:wrf%dom(id)%sn))
   call check( nf90_inq_varid(ncid, "MAPFAC_U", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%mapfac_u) )

   allocate(wrf%dom(id)%mapfac_v(1:wrf%dom(id)%we,1:wrf%dom(id)%sns))
   call check( nf90_inq_varid(ncid, "MAPFAC_V", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%mapfac_v) )

! get 3D base state geopotential

   allocate(wrf%dom(id)%phb(1:wrf%dom(id)%we,1:wrf%dom(id)%sn,1:wrf%dom(id)%bts))
   call check( nf90_inq_varid(ncid, "PHB", var_id) )
   call check( nf90_get_var(ncid, var_id, wrf%dom(id)%phb) )
   if(debug) then
      write(6,*) ' corners of phb '
      write(6,*) wrf%dom(id)%phb(1,1,1),wrf%dom(id)%phb(wrf%dom(id)%we,1,1),  &
           wrf%dom(id)%phb(1,wrf%dom(id)%sn,1),wrf%dom(id)%phb(wrf%dom(id)%we, &
           wrf%dom(id)%sn,1)
      write(6,*) wrf%dom(id)%phb(1,1,wrf%dom(id)%bts), &
           wrf%dom(id)%phb(wrf%dom(id)%we,1,wrf%dom(id)%bts),  &
           wrf%dom(id)%phb(1,wrf%dom(id)%sn,wrf%dom(id)%bts), &
           wrf%dom(id)%phb(wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bts)
   end if

! close data file, we have all we need

   call check( nf90_close(ncid) )

! Initializes the map projection structure to missing values

   call map_init(wrf%dom(id)%proj)

! Populate the map projection structure

   if(wrf%dom(id)%map_proj == 0) then
      proj_code = PROJ_LATLON
   elseif(wrf%dom(id)%map_proj == 1) then
      proj_code = PROJ_LC
   elseif(wrf%dom(id)%map_proj == 2) then
      proj_code = PROJ_PS
   elseif(wrf%dom(id)%map_proj == 3) then
      proj_code = PROJ_MERC
   else
      call error_handler(E_ERR,'static_init_model', &
        'Map projection no supported.', source, revision, revdate)
   endif

   call map_set(proj_code,wrf%dom(id)%latitude(1,1),wrf%dom(id)%longitude(1,1), &
        1.0_r8,1.0_r8,wrf%dom(id)%dx,stdlon,truelat1,truelat2,wrf%dom(id)%proj)

!  build the map into the 1D DART vector for WRF data

   wrf%dom(id)%number_of_wrf_variables = 8 + wrf%dom(id)%n_moist
   if( wrf%dom(id)%surf_obs ) then
      wrf%dom(id)%number_of_wrf_variables = wrf%dom(id)%number_of_wrf_variables + 5
   endif
   allocate(wrf%dom(id)%var_type(wrf%dom(id)%number_of_wrf_variables))
   wrf%dom(id)%var_type(1)  = TYPE_U
   wrf%dom(id)%var_type(2)  = TYPE_V
   wrf%dom(id)%var_type(3)  = TYPE_W
   wrf%dom(id)%var_type(4)  = TYPE_GZ
   wrf%dom(id)%var_type(5)  = TYPE_T
   wrf%dom(id)%var_type(6)  = TYPE_MU
   wrf%dom(id)%var_type(7)  = TYPE_TSLB
   wrf%dom(id)%var_type(8)  = TYPE_TSK
   ind = 8
   if( wrf%dom(id)%n_moist >= 1) then
      ind = ind + 1
      wrf%dom(id)%var_type(ind)  = TYPE_QV
   end if
   if( wrf%dom(id)%n_moist >= 2) then
      ind = ind + 1
      wrf%dom(id)%var_type(ind)  = TYPE_QC
   end if
   if( wrf%dom(id)%n_moist >= 3) then
      ind = ind + 1
      wrf%dom(id)%var_type(ind) = TYPE_QR
   end if
   if( wrf%dom(id)%n_moist >= 4) then
      ind = ind + 1
      wrf%dom(id)%var_type(ind) = TYPE_QI
   end if
   if( wrf%dom(id)%n_moist >= 5) then
      ind = ind + 1
      wrf%dom(id)%var_type(ind) = TYPE_QS
   end if
   if( wrf%dom(id)%n_moist == 6) then
      ind = ind + 1
      wrf%dom(id)%var_type(ind) = TYPE_QG
   end if
   if( wrf%dom(id)%surf_obs ) then
      wrf%dom(id)%var_type(ind + 1) = TYPE_U10
      wrf%dom(id)%var_type(ind + 2) = TYPE_V10
      wrf%dom(id)%var_type(ind + 3) = TYPE_T2
      wrf%dom(id)%var_type(ind + 4) = TYPE_Q2
      wrf%dom(id)%var_type(ind + 5) = TYPE_PS
   end if

! indices into 1D array
   allocate(wrf%dom(id)%var_index(2,wrf%dom(id)%number_of_wrf_variables))
! dimension of variables
   allocate(wrf%dom(id)%var_size(3,wrf%dom(id)%number_of_wrf_variables))

   ind = 1                         ! *** u field ***
   wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%wes
   wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sn
   wrf%dom(id)%var_size(3,ind) = wrf%dom(id)%bt
   dart_index = dart_index + 1
   wrf%dom(id)%var_index(1,ind) = dart_index
   dart_index = dart_index - 1 +  &
        wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
   wrf%dom(id)%var_index(2,ind) = dart_index

   ind = ind + 1                   ! *** v field ***
   wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%we
   wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sns
   wrf%dom(id)%var_size(3,ind) = wrf%dom(id)%bt
   dart_index = dart_index + 1
   wrf%dom(id)%var_index(1,ind) = dart_index
   dart_index = dart_index - 1 +  &
        wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
   wrf%dom(id)%var_index(2,ind) = dart_index

   ind = ind + 1                   ! *** w field ***
   wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%we
   wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sn
   wrf%dom(id)%var_size(3,ind) = wrf%dom(id)%bts
   dart_index = dart_index + 1
   wrf%dom(id)%var_index(1,ind) = dart_index
   dart_index = dart_index - 1 +  &
        wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
   wrf%dom(id)%var_index(2,ind) = dart_index

   ind = ind + 1                   ! *** geopotential field ***
   wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%we
   wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sn
   wrf%dom(id)%var_size(3,ind) = wrf%dom(id)%bts
   dart_index = dart_index + 1
   wrf%dom(id)%var_index(1,ind) = dart_index
   dart_index = dart_index - 1 +  &
        wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
   wrf%dom(id)%var_index(2,ind) = dart_index

   ind = ind + 1                   ! *** theta field ***
   wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%we
   wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sn
   wrf%dom(id)%var_size(3,ind) = wrf%dom(id)%bt
   dart_index = dart_index + 1
   wrf%dom(id)%var_index(1,ind) = dart_index
   dart_index = dart_index - 1 +  &
        wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
   wrf%dom(id)%var_index(2,ind) = dart_index

   ind = ind + 1                   ! *** mu field ***
   wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%we
   wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sn
   wrf%dom(id)%var_size(3,ind) = 1
   dart_index = dart_index + 1
   wrf%dom(id)%var_index(1,ind) = dart_index
   dart_index = dart_index - 1 +  &
        wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
   wrf%dom(id)%var_index(2,ind) = dart_index

   ind = ind + 1                   ! *** tslb field ***
   wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%we
   wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sn
   wrf%dom(id)%var_size(3,ind) = wrf%dom(id)%sls
   dart_index = dart_index + 1
   wrf%dom(id)%var_index(1,ind) = dart_index
   dart_index = dart_index - 1 +  &
        wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
   wrf%dom(id)%var_index(2,ind) = dart_index

   ind = ind + 1                   ! *** tsk field ***
   wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%we
   wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sn
   wrf%dom(id)%var_size(3,ind) = 1
   dart_index = dart_index + 1
   wrf%dom(id)%var_index(1,ind) = dart_index
   dart_index = dart_index - 1 +  &
        wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
   wrf%dom(id)%var_index(2,ind) = dart_index

   do i = 1, wrf%dom(id)%n_moist
      ind = ind + 1                   ! *** moisture field ***
      wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%we
      wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sn
      wrf%dom(id)%var_size(3,ind) = wrf%dom(id)%bt
      dart_index = dart_index + 1
      wrf%dom(id)%var_index(1,ind) = dart_index
      dart_index = dart_index - 1 +  &
           wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
      wrf%dom(id)%var_index(2,ind) = dart_index
   enddo

   if(wrf%dom(id)%surf_obs ) then
      do i = 1, 5
         ind = ind + 1                   ! *** Surface variable ***
         wrf%dom(id)%var_size(1,ind) = wrf%dom(id)%we
         wrf%dom(id)%var_size(2,ind) = wrf%dom(id)%sn
         wrf%dom(id)%var_size(3,ind) = 1
         dart_index = dart_index + 1
         wrf%dom(id)%var_index(1,ind) = dart_index
         dart_index = dart_index - 1 +  &
              wrf%dom(id)%var_size(1,ind)*wrf%dom(id)%var_size(2,ind)*wrf%dom(id)%var_size(3,ind)
         wrf%dom(id)%var_index(2,ind) = dart_index
      enddo
   end if

enddo

wrf%model_size = dart_index
if(debug) write(6,*) ' wrf model size is ',wrf%model_size

contains

  ! Internal subroutine - checks error status after each netcdf, prints
  !                       text message each time an error code is returned.
  subroutine check(istatus)
    integer, intent (in) :: istatus

    if(istatus /= nf90_noerr) call error_handler(E_ERR, 'static_init_model', &
       trim(nf90_strerror(istatus)), source, revision, revdate)

  end subroutine check

end subroutine static_init_model


!#######################################################################

function get_model_size()

integer :: get_model_size

get_model_size = wrf%model_size

end function get_model_size

!#######################################################################

function get_model_time_step()
!------------------------------------------------------------------------
! function get_model_time_step()
!
! Returns the time step of the model. In the long run should be replaced
! by a more general routine that returns details of a general time-stepping
! capability.

type(time_type) :: get_model_time_step

! Need to translate from wrf model timestep (in seconds) to
! DART time increment

get_model_time_step = set_time(nint(wrf%dom(1)%dt), 0)

end function get_model_time_step

!#######################################################################

subroutine get_state_meta_data(index_in, location, var_type_out, id_out)
!---------------------------------------------------------------------
!
! Given an integer index into the state vector structure, returns the
! associated location. This is not a function because the more general
! form of the call has a second intent(out) optional argument kind.
! Maybe a functional form should be added?

integer,             intent(in)  :: index_in
type(location_type), intent(out) :: location
integer, optional,   intent(out) :: var_type_out, id_out

integer  :: var_type
integer  :: index, ip, jp, kp
integer  :: nz, ny, nx
logical  :: var_found
real(r8) :: lon, lat, lev

integer :: i, id
logical, parameter :: debug = .false.
character(len=129) :: errstring

if(debug) then
   write(errstring,*)' index_in = ',index_in
   call error_handler(E_MSG,'get_state_meta_data',errstring,' ',' ',' ')
endif

index = index_in
var_found = .false.

!  first find var_type

if(debug) then
   do id=1,num_domains
      do i=1, wrf%dom(id)%number_of_wrf_variables
         write(errstring,*)' domain, var, var_type(i) = ',id,i,wrf%dom(id)%var_type(i)
         call error_handler(E_MSG,'get_state_meta_data',errstring,' ',' ',' ')
      enddo
   enddo
endif

i = 0
id = 1
do while (.not. var_found)
   i = i + 1
   if(i .gt. wrf%dom(id)%number_of_wrf_variables) then
      i = 1
      if (id < num_domains) then
         id = id + 1
      else
         write(errstring,*)' size of vector ',wrf%model_size
         call error_handler(E_MSG,'get_state_meta_data', errstring, ' ', ' ', ' ')
         write(errstring,*)' dart_index ',index_in
         call error_handler(E_ERR,'get_state_meta_data', 'index out of range', &
              source, revision, revdate)
      end if
   end if
   if( (index .ge. wrf%dom(id)%var_index(1,i) ) .and.  &
        (index .le. wrf%dom(id)%var_index(2,i) )       )  then
      var_found = .true.
      var_type = wrf%dom(id)%var_type(i)
      index = index - wrf%dom(id)%var_index(1,i) + 1
   end if
end do

!  now find i,j,k location.
!  index has been normalized such that it is relative to
!  array starting at (1,1,1)

nx = wrf%dom(id)%var_size(1,i)
ny = wrf%dom(id)%var_size(2,i)
nz = wrf%dom(id)%var_size(3,i)

kp = 1 + (index-1)/(nx*ny)
jp = 1 + (index - (kp-1)*nx*ny - 1)/nx
ip = index - (kp-1)*nx*ny - (jp-1)*nx

if(debug) write(6,*) ' ip, jp, kp for index ',ip,jp,kp,index
if(debug) write(6,*) ' Var type: ',var_type

call get_wrf_horizontal_location( ip, jp, var_type, id, lon, lat )

if( (var_type == type_w ) .or. (var_type == type_gz) ) then
   lev = real(kp) - 0.5_r8 ! This is the index of the vertical
else
   lev = real(kp) ! This is the index of the vertical
endif

if(debug) write(6,*) 'lon, lat, lev: ',lon, lat, lev

! lev is an index here, so which_vert is OK to be hardwired to a 1
location = set_location(lon, lat, lev, 1) 

if(present(var_type_out)) var_type_out = var_type
if(present(id_out)) id_out = id

end subroutine get_state_meta_data

!#######################################################################

subroutine model_interpolate(x, location, obs_kind, obs_val, istatus)

real(r8),            intent(in) :: x(:)
type(location_type), intent(in) :: location
integer,             intent(in) :: obs_kind
real(r8),           intent(out) :: obs_val
integer,            intent(out) :: istatus

logical, parameter  :: debug = .false.
real(r8)            :: xloc, yloc, zloc, xloc_u, yloc_v, xyz_loc(3)
integer             :: which_vert
integer             :: i, i_u, j, j_v, k, k2, i1,i2
real(r8)            :: dx,dy,dz,dxm,dym,dzm,dx_u,dxm_u,dy_v,dym_v
real(r8)            :: a1,utrue,vtrue,ugrid,vgrid
integer             :: in, ii, id

real(r8), dimension(2) :: fld
real(r8), allocatable, dimension(:) :: v_h, v_p

! local vars, used in calculating density, pressure
real(r8)            :: rho1 , rho2 , rho3, rho4
real(r8)            :: pres1, pres2, pres3, pres4, pres

logical  :: dom_found

dom_found = .false.

istatus = 0

xyz_loc(:) = get_location(location)
which_vert = nint(query_location(location,'which_vert'))

id = num_domains
do while (.not. dom_found)

   call latlon_to_ij(wrf%dom(id)%proj, xyz_loc(2), xyz_loc(1), xloc,yloc)

   if ( (xloc >= 1 .and. xloc <= wrf%dom(id)%we .and. &
        yloc >= 1 .and. yloc <= wrf%dom(id)%sn) .or. id == 1 ) then

      dom_found = .true.

   else

      id = id - 1

   endif

end do

if(debug) then

   i = xloc
   j = yloc

   print*,xyz_loc(2), xyz_loc(1), xloc,yloc
   write(6,*) ' corners of lat '
   write(6,*) wrf%dom(id)%latitude(i,j),wrf%dom(id)%latitude(i+1,j),  &
        wrf%dom(id)%latitude(i,j+1), &
        wrf%dom(id)%latitude(i+1,j+1)
   write(6,*) ' corners of long '
   write(6,*) wrf%dom(id)%longitude(i,j),wrf%dom(id)%longitude(i+1,j),  &
        wrf%dom(id)%longitude(i,j+1), &
        wrf%dom(id)%longitude(i+1,j+1)

endif

allocate(v_h(0:wrf%dom(id)%bt), v_p(0:wrf%dom(id)%bt))

call toGrid(xloc,i,dx,dxm)
call toGrid(yloc,j,dy,dym)

if(which_vert == 1) then

   ! If obs is by model level
   zloc = xyz_loc(3)
   if(debug) print*,' obs is by model level and zloc =',zloc
else if(which_vert == 2) then
   ! get model pressure profile
   call get_model_pressure_profile(i,j,dx,dy,dxm,dym,wrf%dom(id)%bt,x,id,v_p)
   ! get pressure vertical co-ordinate
   call pres_to_zk(xyz_loc(3), v_p, wrf%dom(id)%bt,zloc)
   if(debug.and.obs_kind /= KIND_PS) print*,' obs is by pressure and zloc =',zloc
   if(debug) print*,'model pressure profile'
   if(debug) print*,v_p
else if(which_vert == 3) then
   ! get model height profile
   call get_model_height_profile(i,j,dx,dy,dxm,dym,wrf%dom(id)%bt,x,id,v_h)
   ! get height vertical co-ordinate
   call height_to_zk(xyz_loc(3), v_h, wrf%dom(id)%bt,zloc)
   if(debug) print*,' obs is by height and zloc =',zloc
else if(which_vert == -1) then
   ! get terrain height
   v_h(0) = dym*( dxm*wrf%dom(id)%hgt(i,j) + &
                   dx*wrf%dom(id)%hgt(i+1,j) ) + &
             dy*( dxm*wrf%dom(id)%hgt(i,j+1) + &
                   dx*wrf%dom(id)%hgt(i+1,j+1) )
   zloc = 0.0_r8
   if(debug) print*,' obs is at the surface = ', xyz_loc(3)
else
   call error_handler(E_ERR,'model_interpolate', 'wrong option for which_vert', &
        source, revision, revdate)
end if

if(zloc /= missing_r8) then

k = max(1,int(zloc))

! Get the desired field to be interpolated
if( obs_kind == KIND_U .or. obs_kind == KIND_V) then        ! U, V

   xloc_u = xloc + 0.5
   yloc_v = yloc + 0.5
   call toGrid(xloc_u,i_u,dx_u,dxm_u)
   call toGrid(yloc_v,j_v,dy_v,dym_v)

   if(i_u >= 1 .and. i_u < wrf%dom(id)%var_size(1,TYPE_U) .and. &
      j   >= 1 .and. j   < wrf%dom(id)%var_size(2,TYPE_U) .and. &
      i   >= 1 .and. i   < wrf%dom(id)%var_size(1,TYPE_V) .and. &
      j_v >= 1 .and. j_v < wrf%dom(id)%var_size(2,TYPE_V)) then

      do k2=1,2

         i1 = get_wrf_index(i_u,j  ,k+k2-1,TYPE_U,id)
         i2 = get_wrf_index(i_u,j+1,k+k2-1,TYPE_U,id)

         ugrid = dym*( dxm_u*x(i1) + dx_u*x(i1+1) ) + dy*( dxm_u*x(i2) + dx_u*x(i2+1) )

         i1 = get_wrf_index(i,j_v  ,k+k2-1,TYPE_V,id)
         i2 = get_wrf_index(i,j_v+1,k+k2-1,TYPE_V,id) 

         vgrid = dym_v*( dxm*x(i1) + dx*x(i1+1) ) + dy_v*( dxm*x(i2) + dx*x(i2+1) )

         call gridwind_to_truewind(xyz_loc(1), wrf%dom(id)%proj, ugrid, vgrid, &
              utrue, vtrue)

         if( obs_kind == KIND_U) then

            fld(k2) = utrue

         else   ! must want v

            fld(k2) = vtrue

         endif

      enddo

   else

      fld(:) = missing_r8

   endif

else if( obs_kind == KIND_T ) then                ! T

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,TYPE_T) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,TYPE_T)) then

      i1 = get_wrf_index(i,j  ,k,TYPE_T,id)
      i2 = get_wrf_index(i,j+1,k,TYPE_T,id)
      a1 = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )
      pres1 = model_pressure_t(i  ,j  ,k,id,x)
      pres2 = model_pressure_t(i+1,j  ,k,id,x)
      pres3 = model_pressure_t(i  ,j+1,k,id,x)
      pres4 = model_pressure_t(i+1,j+1,k,id,x)
      pres = dym*( dxm*pres1 + dx*pres2 ) + dy*( dxm*pres3 + dx*pres4 )
      fld(1) = (ts0 + a1)*(pres/ps0)**kappa

      i1 = get_wrf_index(i,j  ,k+1,TYPE_T,id)
      i2 = get_wrf_index(i,j+1,k+1,TYPE_T,id)
      a1 = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )
      pres1 = model_pressure_t(i  ,j  ,k+1,id,x)
      pres2 = model_pressure_t(i+1,j  ,k+1,id,x)
      pres3 = model_pressure_t(i  ,j+1,k+1,id,x)
      pres4 = model_pressure_t(i+1,j+1,k+1,id,x)
      pres = dym*( dxm*pres1 + dx*pres2 ) + dy*( dxm*pres3 + dx*pres4 )
      fld(2) = (ts0 + a1)*(pres/ps0)**kappa

   else

      fld(:) = missing_r8

   endif

else if( obs_kind == KIND_RHO ) then

   ! Rho calculated at mass points, and so is like "TYPE_T" 
   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,TYPE_T) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,TYPE_T)) then

      ! calculate full rho at corners of interp box
      ! and interpolate to desired horizontal location

      rho1 = model_rho_t(i  ,j  ,k,id,x)
      rho2 = model_rho_t(i+1,j  ,k,id,x)
      rho3 = model_rho_t(i  ,j+1,k,id,x)
      rho4 = model_rho_t(i+1,j+1,k,id,x)
      fld(1) = dym*( dxm*rho1 + dx*rho2 ) + dy*( dxm*rho3 + dx*rho4 )

      rho1 = model_rho_t(i  ,j  ,k+1,id,x)
      rho2 = model_rho_t(i+1,j  ,k+1,id,x)
      rho3 = model_rho_t(i  ,j+1,k+1,id,x)
      rho4 = model_rho_t(i+1,j+1,k+1,id,x)
      fld(2) = dym*( dxm*rho1 + dx*rho2 ) + dy*( dxm*rho3 + dx*rho4 )

   else

      fld(:) = missing_r8

   endif

else if( obs_kind == KIND_W ) then                ! W

   zloc = zloc + 0.5_r8
   k = max(1,int(zloc))

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,TYPE_W) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,TYPE_W)) then

      i1 = get_wrf_index(i,j  ,k,TYPE_W,id)
      i2 = get_wrf_index(i,j+1,k,TYPE_W,id)
      fld(1) = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

      i1 = get_wrf_index(i,j  ,k+1,TYPE_W,id)
      i2 = get_wrf_index(i,j+1,k+1,TYPE_W,id)
      fld(2) = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

   else

      fld(:) = missing_r8

   endif

else if( obs_kind == KIND_QV ) then                ! QV

   in = 0
   do ii = 1, wrf%dom(id)%number_of_wrf_variables 
      if(TYPE_QV == wrf%dom(id)%var_type(ii) ) in = ii
   enddo

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,in) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,in)) then

      i1 = get_wrf_index(i,j  ,k,TYPE_QV,id)
      i2 = get_wrf_index(i,j+1,k,TYPE_QV,id)
      a1 = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )
      fld(1) = a1 /(1.0_r8 + a1)

      i1 = get_wrf_index(i,j  ,k+1,TYPE_QV,id)
      i2 = get_wrf_index(i,j+1,k+1,TYPE_QV,id)
      a1 = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )
      fld(2) = a1 /(1.0_r8 + a1)

   else

      fld(:) = missing_r8

   endif

else if( obs_kind == KIND_QR) then                ! QR

   in = 0
   do ii = 1, wrf%dom(id)%number_of_wrf_variables 
      if(TYPE_QR == wrf%dom(id)%var_type(ii) ) in = ii
   enddo

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,in) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,in)) then

      i1 = get_wrf_index(i,j  ,k,TYPE_QR,id)
      i2 = get_wrf_index(i,j+1,k,TYPE_QR,id)
      fld(1) = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

      i1 = get_wrf_index(i,j  ,k+1,TYPE_QR,id)
      i2 = get_wrf_index(i,j+1,k+1,TYPE_QR,id)
      fld(2) = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

   else

      fld(:) = missing_r8

   endif

else if( obs_kind == KIND_P) then                 ! Pressure

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,TYPE_T) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,TYPE_T)) then

      pres1 = model_pressure_t(i  ,j  ,k,id,x)
      pres2 = model_pressure_t(i+1,j  ,k,id,x)
      pres3 = model_pressure_t(i  ,j+1,k,id,x)
      pres4 = model_pressure_t(i+1,j+1,k,id,x)
      fld(1) = dym*( dxm*pres1 + dx*pres2 ) + dy*( dxm*pres3 + dx*pres4 )

      pres1 = model_pressure_t(i  ,j  ,k+1,id,x)
      pres2 = model_pressure_t(i+1,j  ,k+1,id,x)
      pres3 = model_pressure_t(i  ,j+1,k+1,id,x)
      pres4 = model_pressure_t(i+1,j+1,k+1,id,x)
      fld(2) = dym*( dxm*pres1 + dx*pres2 ) + dy*( dxm*pres3 + dx*pres4 )

   else

      fld(:) = missing_r8

   endif

else if( obs_kind == KIND_PS) then                ! Surface pressure

   in = 0
   do ii = 1, wrf%dom(id)%number_of_wrf_variables 
      if(TYPE_PS == wrf%dom(id)%var_type(ii) ) in = ii
   enddo

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,in) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,in)) then

      i1 = get_wrf_index(i,j,1,TYPE_PS,id)
      i2 = get_wrf_index(i,j+1,1,TYPE_PS,id)
      obs_val = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

      if(debug) print*,' for sfc model val =',obs_val

   else

      obs_val = missing_r8

   endif

else if( obs_kind == KIND_U10 ) then                ! 10-m U-wind

   in = 0
   do ii = 1, wrf%dom(id)%number_of_wrf_variables 
      if(TYPE_U10 == wrf%dom(id)%var_type(ii) ) in = ii
   enddo

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,in) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,in)) then

      i1 = get_wrf_index(i,j,1,TYPE_U10,id)
      i2 = get_wrf_index(i,j+1,1,TYPE_U10,id)
      obs_val = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

   else

      obs_val = missing_r8

   endif

else if( obs_kind == KIND_V10 ) then                ! 10-m V-wind

   in = 0
   do ii = 1, wrf%dom(id)%number_of_wrf_variables 
      if(TYPE_V10 == wrf%dom(id)%var_type(ii) ) in = ii
   enddo

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,in) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,in)) then

      i1 = get_wrf_index(i,j,1,TYPE_V10,id)
      i2 = get_wrf_index(i,j+1,1,TYPE_V10,id)
      obs_val = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

   else

      obs_val = missing_r8

   endif

else if( obs_kind == KIND_T2 ) then                ! 2-m Temperature

   in = 0
   do ii = 1, wrf%dom(id)%number_of_wrf_variables 
      if(TYPE_T2 == wrf%dom(id)%var_type(ii) ) in = ii
   enddo

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,in) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,in)) then

      i1 = get_wrf_index(i,j,1,TYPE_T2,id)
      i2 = get_wrf_index(i,j+1,1,TYPE_T2,id)
      obs_val = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

   else

      obs_val = missing_r8

   endif

else if( obs_kind == KIND_Q2 ) then                ! 2-m Specific humidity

   in = 0
   do ii = 1, wrf%dom(id)%number_of_wrf_variables 
      if(TYPE_Q2 == wrf%dom(id)%var_type(ii) ) in = ii
   enddo

   if(i >= 1 .and. i < wrf%dom(id)%var_size(1,in) .and. &
      j >= 1 .and. j < wrf%dom(id)%var_size(2,in)) then

      i1 = get_wrf_index(i,j,1,TYPE_Q2,id)
      i2 = get_wrf_index(i,j+1,1,TYPE_Q2,id)
      obs_val = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

   else

      obs_val = missing_r8

   endif

else
   call error_handler(E_ERR,'model_interpolate', 'wrong obs kind', &
        source, revision, revdate)
end if

else

   obs_val = missing_r8
   fld(:) = missing_r8

endif

! Do vertical interpolation
if(obs_kind /= KIND_PS .and. obs_kind /= KIND_U10 .and. obs_kind /= KIND_V10 &
     .and. obs_kind /= KIND_T2 .and. obs_kind /= KIND_Q2) then

   call toGrid(zloc, k, dz, dzm)

   if( k >= 1 ) then
      obs_val = dzm*fld(1) + dz*fld(2)
   elseif( k == 0 ) then
      obs_val = fld(1) - (fld(2)-fld(1))*dzm
   else
      obs_val = missing_r8
   endif
endif

if(obs_val == missing_r8) istatus = 1

if(debug) print*,' interpolated value= ',obs_val

deallocate(v_h, v_p)

end subroutine model_interpolate

!#######################################################################

subroutine model_get_close_states(o_loc, radius, num_total, indices, dist, x)

type(location_type), intent(in) :: o_loc
real(r8),            intent(in) :: radius
integer,            intent(out) :: num_total, indices(:)
real(r8),           intent(out) :: dist(:)
real(r8),            intent(in) :: x(:)

integer                         :: u_pts, v_pts, p_pts, w_pts
integer                         :: i,k,indmax, ii, jj, kk, id
integer                         :: max_size
integer,            allocatable :: lon_ind(:), lat_ind(:), vert_ind(:)
real(r8),           allocatable :: close_dist(:)

indmax = size(indices)

num_total = 0

do id=1,num_domains

! We're allocating enough space for all u, v, w, and p points.

   max_size = (wrf%dom(id)%we*wrf%dom(id)%sn + wrf%dom(id)%wes*wrf%dom(id)%sn + &
        wrf%dom(id)%we*(wrf%dom(id)%sns))*wrf%dom(id)%bt + &
        wrf%dom(id)%we*wrf%dom(id)%sn*wrf%dom(id)%bts
   allocate(lon_ind(max_size), lat_ind(max_size), vert_ind(max_size), &
        close_dist(max_size))

! Look for close grid points.
   call grid_close_states( o_loc, &
        radius, id, lon_ind, lat_ind, vert_ind, close_dist, &
        u_pts, v_pts, p_pts, w_pts, x )

! start with u_pts

   do i = 1, u_pts

      ii = lon_ind(i)
      jj = lat_ind(i)
      k = vert_ind(i)

      num_total = num_total + 1
      if(num_total <= indmax) then
         indices(num_total) = get_wrf_index(ii,jj,k,type_u,id)
         dist(num_total) = close_dist(i)
      end if

   enddo

! now v_pts

   do i = u_pts+1, u_pts+v_pts

      ii = lon_ind(i)
      jj = lat_ind(i)
      k = vert_ind(i)

      num_total = num_total + 1
      if(num_total <= indmax) then
         indices(num_total) = get_wrf_index(ii,jj,k,type_v,id)
         dist(num_total) = close_dist(i)
      end if

   enddo

! now w_pts (w and gz)

   do i = u_pts+v_pts+1, u_pts+v_pts+w_pts

      ii = lon_ind(i)
      jj = lat_ind(i)
      k = vert_ind(i)

      num_total = num_total + 1
      if(num_total <= indmax) then
         indices(num_total) = get_wrf_index(ii,jj,k,type_w,id)
         dist(num_total) = close_dist(i)
      end if
      num_total = num_total + 1
      if(num_total <= indmax) then
         indices(num_total) = get_wrf_index(ii,jj,k,type_gz,id)
         dist(num_total) = close_dist(i)
      end if

! 2D (near-surface) fields.

      if (k == 1) then

         num_total = num_total + 1
         if(num_total <= indmax) then
            indices(num_total) = get_wrf_index(ii,jj,k,type_mu,id)
            dist(num_total) = close_dist(i)
         end if

         num_total = num_total + 1
         if(num_total <= indmax) then
            indices(num_total) = get_wrf_index(ii,jj,k,type_tsk,id)
            dist(num_total) = close_dist(i)
         end if

         do kk = 1, wrf%dom(id)%sls
            num_total = num_total + 1
            if(num_total <= indmax) then
               indices(num_total) = get_wrf_index(ii,jj,kk,type_tslb,id)
               dist(num_total) = close_dist(i)
            end if
         enddo

         if(wrf%dom(id)%surf_obs ) then

            num_total = num_total + 1
            if(num_total <= indmax) then
               indices(num_total) = get_wrf_index(ii,jj,k,type_u10,id)
               dist(num_total) = close_dist(i)
            end if

            num_total = num_total + 1
            if(num_total <= indmax) then
               indices(num_total) = get_wrf_index(ii,jj,k,type_v10,id)
               dist(num_total) = close_dist(i)
            end if

            num_total = num_total + 1
            if(num_total <= indmax) then
               indices(num_total) = get_wrf_index(ii,jj,k,type_t2,id)
               dist(num_total) = close_dist(i)
            end if

            num_total = num_total + 1
            if(num_total <= indmax) then
               indices(num_total) = get_wrf_index(ii,jj,k,type_q2,id)
               dist(num_total) = close_dist(i)
            end if

            num_total = num_total + 1
            if(num_total <= indmax) then
               indices(num_total) = get_wrf_index(ii,jj,k,type_ps,id)
               dist(num_total) = close_dist(i)
            end if

         end if

      end if

   enddo

! last -> p_pts (t + wrf%dom(id)%n_moist variables)

   do i = u_pts+v_pts+w_pts+1, u_pts+v_pts+w_pts+p_pts

      ii = lon_ind(i)
      jj = lat_ind(i)
      k = vert_ind(i)

      num_total = num_total + 1
      if(num_total <= indmax) then
         indices(num_total) = get_wrf_index(ii,jj,k,type_t,id)
         dist(num_total) = close_dist(i)
      end if
      if( wrf%dom(id)%n_moist >= 1) then
         num_total = num_total + 1
         if(num_total <= indmax) then
            indices(num_total) = get_wrf_index(ii,jj,k,type_qv,id)
            dist(num_total) = close_dist(i)
         end if
      end if
      if( wrf%dom(id)%n_moist >= 2) then
         num_total = num_total + 1
         if(num_total <= indmax) then
            indices(num_total) = get_wrf_index(ii,jj,k,type_qc,id)
            dist(num_total) = close_dist(i)
         end if
      end if
      if( wrf%dom(id)%n_moist >= 3) then
         num_total = num_total + 1
         if(num_total <= indmax) then
            indices(num_total) = get_wrf_index(ii,jj,k,type_qr,id)
            dist(num_total) = close_dist(i)
         end if
      end if
      if( wrf%dom(id)%n_moist >= 4) then
         num_total = num_total + 1
         if(num_total <= indmax) then
            indices(num_total) = get_wrf_index(ii,jj,k,type_qi,id)
            dist(num_total) = close_dist(i)
         end if
      end if
      if( wrf%dom(id)%n_moist >= 5) then
         num_total = num_total + 1
         if(num_total <= indmax) then
            indices(num_total) = get_wrf_index(ii,jj,k,type_qs,id)
            dist(num_total) = close_dist(i)
         end if
      end if
      if( wrf%dom(id)%n_moist == 6) then
         num_total = num_total + 1
         if(num_total <= indmax) then
            indices(num_total) = get_wrf_index(ii,jj,k,type_qg,id)
            dist(num_total) = close_dist(i)
         end if
      end if

   enddo

   deallocate(lon_ind, lat_ind, vert_ind, close_dist)

enddo

end subroutine model_get_close_states

!-----------------------------------------------------------------

function get_wrf_index( i,j,k,var_type,id )

integer, intent(in) :: i,j,k,var_type,id

integer :: get_wrf_index
integer :: in, ii

character(len=129) :: errstring

in = 0
do ii = 1, wrf%dom(id)%number_of_wrf_variables 
   if(var_type == wrf%dom(id)%var_type(ii) ) in = ii
enddo

if(i >= 1 .and. i <= wrf%dom(id)%var_size(1,in) .and. &
   j >= 1 .and. j <= wrf%dom(id)%var_size(2,in) .and. &
   k >= 1 .and. k <= wrf%dom(id)%var_size(3,in)) then

   get_wrf_index = wrf%dom(id)%var_index(1,in)-1 +   &
        i + wrf%dom(id)%var_size(1,in)*(j-1) + &
        (wrf%dom(id)%var_size(1,in)*wrf%dom(id)%var_size(2,in))*(k-1)

else

  write(errstring,*)'Indices ',i,j,k,' exceed grid dimensions: ', &
       wrf%dom(id)%var_size(1,in), &
       wrf%dom(id)%var_size(2,in),wrf%dom(id)%var_size(3,in)
  call error_handler(E_ERR,'get_wrf_index', errstring, &
       source, revision, revdate)

endif

end function get_wrf_index

!-----------------------------------------------------------------

subroutine grid_close_states( o_loc, radius_in, id,  &
                              close_lon_ind, close_lat_ind, close_vert_ind, &
                              close_dist, u_pts, v_pts, p_pts, w_pts, x )

! Finds close state points from a particular grid for the WRF model

type(location_type), intent(in) :: o_loc
real(r8),            intent(in) :: radius_in
integer,             intent(in) :: id
integer,            intent(out) :: close_lon_ind(:), close_lat_ind(:), close_vert_ind(:)
real(r8),           intent(out) :: close_dist(:)
integer,            intent(out) :: u_pts, v_pts, p_pts, w_pts
real(r8),            intent(in) :: x(:)

real(r8)            :: radius
real(r8)            :: rad, radn, dxr, dyr, sdx, sdy, gdist
integer             :: i_closest, j_closest, ixmin, jymin, ixmax, jymax

integer             :: i, j, k, n, m, fis, is, ie, num

logical,  parameter :: debug = .false.  

if(debug) write(6,*) ' in grid_close_states '

! use meaningful units for radius -- convert radians to meters 
radius = radius_in*earth_radius*1000.0_r8
if(debug) write(6,*) ' radius in grid_close_states is ',radius

! Get index to closest lat and lon for this observation

n = wrf%dom(id)%we
m = wrf%dom(id)%sn

if(debug) write(6,*) 'obs location: ',get_location(o_loc)

rad = get_dist_wrf(1,1,1, type_t, o_loc, id, x)
i_closest = 1
j_closest = 1

! brute force search
do j=1,m
   do i=1,n
      radn = get_dist_wrf(i,j,1, type_t, o_loc, id, x)
      if( radn .lt. rad ) then
         rad = radn
         i_closest = i
         j_closest = j
      end if
      if(debug) write(6,*) i,j,radn,rad
   enddo
enddo

if(debug) write(6,*) ' closest grid point is ',i_closest,j_closest
if(debug) write(6,*) ' at distance ',rad

! define box edges for radius check
dxr = 1.0_r8 + radius/wrf%dom(id)%dx  !  radius in multiples of dx
dyr = 1.0_r8 + radius/wrf%dom(id)%dy  !  radius in multiples of dy

if(debug) write(6,*) ' dxr, dyr in grid_close_states ',dxr,dyr


j = j_closest
ixmin = max(1,i_closest - 1)
sdx   = 1.0_r8/wrf%dom(id)%mapfac_u(i_closest, j)
do while( sdx .lt. dxr )
   ixmin = max(1,ixmin - 1)
   sdx = sdx + 1.0_r8/wrf%dom(id)%mapfac_u(ixmin + 1, j)
   if(ixmin <= 1) sdx = 1.1_r8*dxr
enddo

ixmax = min(wrf%dom(id)%we,i_closest + 1)
sdx   = 1.0_r8/wrf%dom(id)%mapfac_u(ixmax, j)
do while( sdx .lt. dxr )
   ixmax = min(wrf%dom(id)%we,ixmax + 1)
   sdx = sdx + 1.0_r8/wrf%dom(id)%mapfac_u(ixmax, j)
   if(ixmax >= wrf%dom(id)%we) sdx = 1.1_r8*dxr
enddo

i = i_closest
jymin = max(1,j_closest - 1)
sdy   = 1.0_r8/wrf%dom(id)%mapfac_u(i_closest, jymin)
do while( sdy .lt. dyr )
   jymin = max(1,jymin - 1)
   sdy = sdy + 1.0_r8/wrf%dom(id)%mapfac_u(i, jymin + 1)
   if(jymin <= 1) sdy = 1.1_r8*dyr
enddo

jymax = min(wrf%dom(id)%sn,j_closest + 1)
sdy   = 1.0_r8/wrf%dom(id)%mapfac_u(i, jymax)
do while( sdy .lt. dyr )
   jymax = min(wrf%dom(id)%sn,jymax + 1)
   sdy = sdy + 1.0_r8/wrf%dom(id)%mapfac_u(i, jymax)
   if(jymax >= wrf%dom(id)%sn) sdy = 1.1_r8*dyr
enddo

if(debug) then
   write(6,*) ' ixmin, ixmax, jymin, jymax are '
   write(6,*) ixmin, ixmax, jymin, jymax
endif

!  we have bounding box, get and check distances.
!  first, convert radius back to radians

radius = radius_in
num = 0

if(debug) write(6,*) 'radius (radian) ',radius

! check distance for u points, expand box so that
! we don't leave possible points out of check

k = 1
do j = jymin, jymax
   do i = max(1,ixmin-1), ixmax+1

      gdist = get_dist_wrf(i,j,k, type_u, o_loc, id, x)
      if ( gdist <= radius ) then
         num = num + 1
         if(debug) write(6,*) ' u pt ',num,i,j,gdist
         close_lon_ind(num) = i
         close_lat_ind(num) = j
         close_vert_ind(num) = k
         close_dist(num) = gdist
      end if

   enddo
enddo

u_pts = num

if(horiz_dist_only) then

   fis = 1
   is = num + 1
   ie = num + u_pts
   do k = 2, wrf%dom(id)%bt
      close_lon_ind(is:ie) = close_lon_ind(fis:num)
      close_lat_ind(is:ie) = close_lat_ind(fis:num)
      close_vert_ind(is:ie) = k
      close_dist(is:ie) = close_dist(fis:num)
      is = is + u_pts
      ie = ie + u_pts
   enddo

   num = num + u_pts*(wrf%dom(id)%bt - 1)

else

   do k = 2, wrf%dom(id)%bt
      do j = jymin, jymax
         do i = max(1,ixmin-1), ixmax+1

            gdist = get_dist_wrf(i,j,k, type_u, o_loc, id, x)
            if ( gdist <= radius ) then
               num = num + 1
               if(debug) write(6,*) ' u pt ',num,i,j,k, id,gdist
               close_lon_ind(num) = i
               close_lat_ind(num) = j
               close_vert_ind(num) = k
               close_dist(num) = gdist
            end if

         enddo
      enddo
   enddo

endif

u_pts = num

k = 1
do j = max(1,jymin-1), jymax+1
   do i = ixmin, ixmax

      gdist = get_dist_wrf(i,j,k, type_v, o_loc, id, x)
      if ( gdist <= radius ) then
         num = num + 1
         close_lon_ind(num) = i
         close_lat_ind(num) = j
         close_vert_ind(num) = k
         close_dist(num) = gdist
         if(debug) write(6,*) ' v pt ',num,i,j,gdist
      end if
   enddo
enddo

v_pts = num - u_pts

if(horiz_dist_only) then

   fis = u_pts + 1
   is = num + 1
   ie = num + v_pts
   do k = 2, wrf%dom(id)%bt
      close_lon_ind(is:ie) = close_lon_ind(fis:num)
      close_lat_ind(is:ie) = close_lat_ind(fis:num)
      close_vert_ind(is:ie) = k
      close_dist(is:ie) = close_dist(fis:num)
      is = is + v_pts
      ie = ie + v_pts
   enddo

   num = num + v_pts*(wrf%dom(id)%bt - 1)

else

   do k = 2, wrf%dom(id)%bt
      do j = max(1,jymin-1), jymax+1
         do i = ixmin, ixmax

            gdist = get_dist_wrf(i,j,k, type_v, o_loc, id, x)
            if ( gdist <= radius ) then
               num = num + 1
               close_lon_ind(num) = i
               close_lat_ind(num) = j
               close_vert_ind(num) = k
               close_dist(num) = gdist
               if(debug) write(6,*) ' v pt ',num,i,j,gdist
            end if
         enddo
      enddo
   enddo

endif

v_pts = num - u_pts

k = 1
do j = jymin, jymax
   do i = ixmin, ixmax

      gdist = get_dist_wrf(i,j,k, type_w, o_loc, id, x)
      if ( gdist <= radius ) then
         num = num + 1
         close_lon_ind(num) = i
         close_lat_ind(num) = j
         close_vert_ind(num) = k
         close_dist(num) = gdist
         if(debug) write(6,*) ' w pt ',num,i,j,gdist
      end if
   enddo
enddo

w_pts = num - u_pts - v_pts

if(horiz_dist_only) then

   fis = u_pts + v_pts + 1
   is = num + 1
   ie = num + w_pts
   do k = 2, wrf%dom(id)%bts
      close_lon_ind(is:ie) = close_lon_ind(fis:num)
      close_lat_ind(is:ie) = close_lat_ind(fis:num)
      close_vert_ind(is:ie) = k
      close_dist(is:ie) = close_dist(fis:num)
      is = is + w_pts
      ie = ie + w_pts
   enddo

   num = num + w_pts*(wrf%dom(id)%bts - 1)

else

   do k = 2, wrf%dom(id)%bts
      do j = jymin, jymax
         do i = ixmin, ixmax

            gdist = get_dist_wrf(i,j,k, type_w, o_loc, id, x)
            if ( gdist <= radius ) then
               num = num + 1
               close_lon_ind(num) = i
               close_lat_ind(num) = j
               close_vert_ind(num) = k
               close_dist(num) = gdist
               if(debug) write(6,*) ' w pt ',num,i,j,gdist
            end if
         enddo
      enddo
   enddo

endif

w_pts = num - u_pts - v_pts

k = 1
do j = jymin, jymax
   do i = ixmin, ixmax

      gdist = get_dist_wrf(i,j,k, type_t, o_loc, id, x)
      if ( gdist <= radius ) then
         num = num + 1
         close_lon_ind(num) = i
         close_lat_ind(num) = j
         close_vert_ind(num) = k
         close_dist(num) = gdist
         if(debug) write(6,*) ' p pt ',num,i,j,gdist
      end if

   enddo
enddo

p_pts = num - u_pts - v_pts - w_pts

if(horiz_dist_only) then

   fis = u_pts + v_pts + w_pts + 1
   is = num + 1
   ie = num + p_pts
   do k = 2, wrf%dom(id)%bt
      close_lon_ind(is:ie) = close_lon_ind(fis:num)
      close_lat_ind(is:ie) = close_lat_ind(fis:num)
      close_vert_ind(is:ie) = k
      close_dist(is:ie) = close_dist(fis:num)
      is = is + p_pts
      ie = ie + p_pts
   enddo

   num = num + p_pts*(wrf%dom(id)%bt - 1)

else

   do k = 2, wrf%dom(id)%bt
      do j = jymin, jymax
         do i = ixmin, ixmax

            gdist = get_dist_wrf(i,j,k, type_t, o_loc, id, x)
            if ( gdist <= radius ) then
               num = num + 1
               close_lon_ind(num) = i
               close_lat_ind(num) = j
               close_vert_ind(num) = k
               close_dist(num) = gdist
               if(debug) write(6,*) ' p pt ',num,i,j,gdist
            end if

         enddo
      enddo
   enddo

endif

p_pts = num - u_pts - v_pts - w_pts

end subroutine grid_close_states

!***********************************************************************

function get_dist_wrf( i, j, k, var_type, o_loc, id, x )

real(r8)                        :: get_dist_wrf 
type(location_type), intent(in) :: o_loc
integer,             intent(in) :: i,j,k,var_type, id
real(r8),            intent(in) :: x(:)

type(location_type) :: loc
real(r8)            :: long, lat, vloc
integer             :: which_vert

character(len=129) :: errstring

! get distance for input var_type

call get_wrf_horizontal_location( i, j, var_type, id, long, lat )

! We will set which_vert based on the incoming location type
! This will ensure compatibility for measuring 'horizontal'
! and cannot think of anything better for vertical.

which_vert = nint(query_location(o_loc,'which_vert'))

if(horiz_dist_only .or. which_vert == -2) then

   vloc = missing_r8

else

   if(which_vert == 1 ) then

      if( (var_type == type_w ) .or. (var_type == type_gz) ) then
         vloc = real(k) - 0.5_r8
      else
         vloc = real(k)
      endif

   elseif(which_vert == 2 ) then

      vloc = model_pressure(i,j,k,id,var_type,x)

   elseif(which_vert == 3 ) then

      vloc = model_height(i,j,k,id,var_type,x)

   elseif(which_vert == -1 ) then

      vloc = wrf%dom(id)%hgt(i,j)

   else
      write(errstring, *) 'Which_vert = ',which_vert, ' is not allowed.'
      call error_handler(E_ERR,'get_dist_wrf', errstring, source, revision, revdate)
   endif

   if (vloc == missing_r8 ) then
      print*,i,j,k, id, var_type,which_vert
      write(errstring, *) 'Unable to define vloc.'
      call error_handler(E_ERR,'get_dist_wrf', errstring, source, revision, revdate)
   endif

endif

loc          = set_location(long, lat, vloc, which_vert)
get_dist_wrf = get_dist(loc,o_loc)

end function get_dist_wrf



subroutine get_wrf_horizontal_location( i, j, var_type, id, long, lat )

integer,  intent(in)  :: i,j,var_type, id
real(r8), intent(out) :: long, lat

! find lat and long, must
! correct for possible u or v staggering in x, y

if(var_type == type_u) then

   if (i == 1) then
      long = wrf%dom(id)%longitude(1,j) - &
           0.5_r8*(wrf%dom(id)%longitude(2,j)-wrf%dom(id)%longitude(1,j))
      if ( abs(wrf%dom(id)%longitude(2,j) - wrf%dom(id)%longitude(1,j)) > 180.0_r8 ) then
         long = long - 180.0_r8
      endif
      lat = wrf%dom(id)%latitude(1,j) - &
           0.5_r8*(wrf%dom(id)%latitude(2,j)-wrf%dom(id)%latitude(1,j))
   else if (i == wrf%dom(id)%wes) then
      long = wrf%dom(id)%longitude(i-1,j) + &
           0.5_r8*(wrf%dom(id)%longitude(i-1,j)-wrf%dom(id)%longitude(i-2,j))
      if ( abs(wrf%dom(id)%longitude(i-1,j) - wrf%dom(id)%longitude(i-2,j)) > 180.0_r8 ) then
         long = long - 180.0_r8
      endif
      lat = wrf%dom(id)%latitude(i-1,j) + &
           0.5_r8*(wrf%dom(id)%latitude(i-1,j)-wrf%dom(id)%latitude(i-2,j))
   else
      long = 0.5_r8*(wrf%dom(id)%longitude(i,j)+wrf%dom(id)%longitude(i-1,j))
      if ( abs(wrf%dom(id)%longitude(i,j) - wrf%dom(id)%longitude(i-1,j)) > 180.0_r8 ) then
         long = long - 180.0_r8
      endif
      lat = 0.5_r8*(wrf%dom(id)%latitude(i,j) +wrf%dom(id)%latitude(i-1,j))
   end if

else if( var_type == type_v) then

   if (j == 1) then
      long = wrf%dom(id)%longitude(i,1) - &
           0.5_r8*(wrf%dom(id)%longitude(i,2)-wrf%dom(id)%longitude(i,1))
      if ( abs(wrf%dom(id)%longitude(i,2) - wrf%dom(id)%longitude(i,1)) > 180.0_r8 ) then
         long = long - 180.0_r8
      endif
      lat = wrf%dom(id)%latitude(i,1) - &
           0.5_r8*(wrf%dom(id)%latitude(i,2)-wrf%dom(id)%latitude(i,1))
   else if (j == wrf%dom(id)%sns) then
      long = wrf%dom(id)%longitude(i,j-1) + &
           0.5_r8*(wrf%dom(id)%longitude(i,j-1)-wrf%dom(id)%longitude(i,j-2))
      if ( abs(wrf%dom(id)%longitude(i,j-1) - wrf%dom(id)%longitude(i,j-2)) > 180.0_r8 ) then
         long = long - 180.0_r8
      endif
      lat = wrf%dom(id)%latitude(i,j-1) + &
           0.5_r8*(wrf%dom(id)%latitude(i,j-1)-wrf%dom(id)%latitude(i,j-2))
   else
      long = 0.5_r8*(wrf%dom(id)%longitude(i,j)+wrf%dom(id)%longitude(i,j-1))
      if ( abs(wrf%dom(id)%longitude(i,j) - wrf%dom(id)%longitude(i,j-1)) > 180.0_r8 ) then
         long = long - 180.0_r8
      endif
      lat  = 0.5_r8*(wrf%dom(id)%latitude(i,j) +wrf%dom(id)%latitude(i,j-1))

   end if

else

   long = wrf%dom(id)%longitude(i,j)
   lat  = wrf%dom(id)%latitude(i,j)

end if

do while (long <   0.0_r8)
   long = long + 360.0_r8
end do
do while (long > 360.0_r8)
   long = long - 360.0_r8
end do

end subroutine get_wrf_horizontal_location



function nc_write_model_atts( ncFileID ) result (ierr)
!-----------------------------------------------------------------
! Writes the model-specific attributes to a netCDF file
! A. Caya May 7 2003
! T. Hoar Mar 8 2004 writes prognostic flavor

integer, intent(in)  :: ncFileID      ! netCDF file identifier
integer              :: ierr          ! return value of function

!-----------------------------------------------------------------

integer :: nDimensions, nVariables, nAttributes, unlimitedDimID
integer :: StateVarDimID, StateVarVarID, StateVarID, TimeDimID

integer, dimension(num_domains) :: weDimID, weStagDimID, snDimID, snStagDimID, &
     btDimID, btStagDimID, slSDimID

integer :: MemberDimID, DomDimID
integer :: DXVarID, DYVarID, TRUELAT1VarID, TRUELAT2VarID
integer :: CEN_LATVarID, CEN_LONVarID, MAP_PROJVarID

integer, dimension(num_domains) :: DNVarID, ZNUVarID, DNWVarID, phbVarID, &
     MubVarID, LonVarID, LatVarID, ilevVarID, XlandVarID, &
     MapFacMVarID, MapFacUVarID, MapFacVVarID, hgtVarID

integer :: var_id
integer :: i, id

character(len=129) :: errstring

character(len=8)      :: crdate      ! needed by F90 DATE_AND_TIME intrinsic
character(len=10)     :: crtime      ! needed by F90 DATE_AND_TIME intrinsic
character(len=5)      :: crzone      ! needed by F90 DATE_AND_TIME intrinsic
integer, dimension(8) :: values      ! needed by F90 DATE_AND_TIME intrinsic
character(len=NF90_MAX_NAME) :: str1

character (len=1)     :: idom

!-----------------------------------------------------------------

ierr = 0     ! assume normal termination

!-----------------------------------------------------------------
! make sure ncFileID refers to an open netCDF file, 
! and then put into define mode.
!-----------------------------------------------------------------

call check(nf90_Inquire(ncFileID, nDimensions, nVariables, nAttributes, unlimitedDimID))
call check(nf90_Redef(ncFileID))

!-----------------------------------------------------------------
! We need the dimension ID for the number of copies 
!-----------------------------------------------------------------

call check(nf90_inq_dimid(ncid=ncFileID, name="copy", dimid=MemberDimID))
call check(nf90_inq_dimid(ncid=ncFileID, name="time", dimid=  TimeDimID))

if ( TimeDimID /= unlimitedDimId ) then
   write(errstring,*)'Time Dimension ID ',TimeDimID, &
        ' must match Unlimited Dimension ID ',unlimitedDimID
   call error_handler(E_ERR,'nc_write_model_atts', errstring, source, revision, revdate)
endif

!-----------------------------------------------------------------
! Define the model size, state variable dimension ... whatever ...
!-----------------------------------------------------------------
call check(nf90_def_dim(ncid=ncFileID, name="StateVariable", &
                        len=wrf%model_size, dimid = StateVarDimID))

!-----------------------------------------------------------------
! Write Global Attributes 
!-----------------------------------------------------------------
call DATE_AND_TIME(crdate,crtime,crzone,values)
write(str1,'(''YYYY MM DD HH MM SS = '',i4,5(1x,i2.2))') &
                  values(1), values(2), values(3), values(5), values(6), values(7)

call check(nf90_put_att(ncFileID, NF90_GLOBAL, "creation_date",str1))
call check(nf90_put_att(ncFileID, NF90_GLOBAL, "model_source",source))
call check(nf90_put_att(ncFileID, NF90_GLOBAL, "model_revision",revision))
call check(nf90_put_att(ncFileID, NF90_GLOBAL, "model_revdate",revdate))

! how about namelist input? might be nice to save ...

!-----------------------------------------------------------------
! Define the dimensions IDs
!-----------------------------------------------------------------

call check(nf90_def_dim(ncid=ncFileID, name="domain",           &
          len = num_domains,  dimid = DomDimID))

do id=1,num_domains
   write( idom , '(I1)') id
   call check(nf90_def_dim(ncid=ncFileID, name="west_east_d0"//idom,        &
        len = wrf%dom(id)%we,  dimid = weDimID(id)))
   call check(nf90_def_dim(ncid=ncFileID, name="west_east_stag_d0"//idom,   &
        len = wrf%dom(id)%wes, dimid = weStagDimID(id)))
   call check(nf90_def_dim(ncid=ncFileID, name="south_north_d0"//idom,      &
        len = wrf%dom(id)%sn,  dimid = snDimID(id)))
   call check(nf90_def_dim(ncid=ncFileID, name="south_north_stag_d0"//idom, &
        len = wrf%dom(id)%sns, dimid = snStagDimID(id)))
   call check(nf90_def_dim(ncid=ncFileID, name="bottom_top_d0"//idom,       &
        len = wrf%dom(id)%bt,  dimid = btDimID(id)))
   call check(nf90_def_dim(ncid=ncFileID, name="bottom_top_stag_d0"//idom,  &
        len = wrf%dom(id)%bts, dimid = btStagDimID(id)))
   call check(nf90_def_dim(ncid=ncFileID, name="soil_layers_stag_d0"//idom,  &
        len = wrf%dom(id)%sls, dimid = slSDimID(id)))
enddo

!-----------------------------------------------------------------
! Create the (empty) Variables and the Attributes
!-----------------------------------------------------------------

!-----------------------------------------------------------------
! Create the (empty) static variables and their attributes
! Commented block is from wrfinput
!-----------------------------------------------------------------

call check(nf90_def_var(ncFileID, name="DX", xtype=nf90_real, &
     dimids= DomDimID, varid=DXVarID) )
call check(nf90_put_att(ncFileID, DXVarID, "long_name", &
     "X HORIZONTAL RESOLUTION"))
call check(nf90_put_att(ncFileID, DXVarID, "units", "m"))

call check(nf90_def_var(ncFileID, name="DY", xtype=nf90_real, &
     dimids= DomDimID, varid=DYVarID) )
call check(nf90_put_att(ncFileID, DYVarID, "long_name", &
     "Y HORIZONTAL RESOLUTION"))
call check(nf90_put_att(ncFileID, DYVarID, "units", "m"))

call check(nf90_def_var(ncFileID, name="TRUELAT1", xtype=nf90_real, &
     dimids= DomDimID, varid=TRUELAT1VarID) )
call check(nf90_put_att(ncFileID, TRUELAT1VarID, "long_name", &
     "first standard parallel"))
call check(nf90_put_att(ncFileID, TRUELAT1VarID, "units", &
     "degrees, negative is south"))

call check(nf90_def_var(ncFileID, name="TRUELAT2", xtype=nf90_real, &
     dimids= DomDimID, varid=TRUELAT2VarID) )
call check(nf90_put_att(ncFileID, TRUELAT2VarID, "long_name", &
     "second standard parallel"))
call check(nf90_put_att(ncFileID, TRUELAT2VarID, "units", &
     "degrees, negative is south"))

call check(nf90_def_var(ncFileID, name="CEN_LAT", xtype=nf90_real, &
     dimids= DomDimID, varid=CEN_LATVarID) )
call check(nf90_put_att(ncFileID, CEN_LATVarID, "long_name", &
     "center latitude"))
call check(nf90_put_att(ncFileID, CEN_LATVarID, "units", &
     "degrees, negative is south"))

call check(nf90_def_var(ncFileID, name="CEN_LON", xtype=nf90_real, &
     dimids= DomDimID, varid=CEN_LONVarID) )
call check(nf90_put_att(ncFileID, CEN_LONVarID, "long_name", &
     "central longitude"))
call check(nf90_put_att(ncFileID, CEN_LONVarID, "units", &
     "degrees, negative is west"))

call check(nf90_def_var(ncFileID, name="MAP_PROJ", xtype=nf90_real, &
     dimids= DomDimID, varid=MAP_PROJVarID) )
call check(nf90_put_att(ncFileID, MAP_PROJVarID, "long_name", &
     "domain map projection"))
call check(nf90_put_att(ncFileID, MAP_PROJVarID, "units", &
     "0=none, 1=Lambert, 2=polar, 3=Mercator"))

do id=1,num_domains
   write( idom , '(I1)') id

   call check(nf90_def_var(ncFileID, name="DN_d0"//idom, xtype=nf90_real, &
        dimids= btDimID(id), varid=DNVarID(id)) )
   call check(nf90_put_att(ncFileID, DNVarID(id), "long_name", &
        "dn values on half (mass) levels"))
   call check(nf90_put_att(ncFileID, DNVarID(id), "units", "dimensionless"))

   call check(nf90_def_var(ncFileID, name="ZNU_d0"//idom, xtype=nf90_real, &
        dimids= btDimID(id), varid=ZNUVarID(id)) )
   call check(nf90_put_att(ncFileID, ZNUVarID(id), "long_name", &
        "eta values on half (mass) levels"))
   call check(nf90_put_att(ncFileID, ZNUVarID(id), "units", "dimensionless"))

   call check(nf90_def_var(ncFileID, name="DNW_d0"//idom, xtype=nf90_real, &
        dimids= btDimID(id), varid=DNWVarID(id)) )
   call check(nf90_put_att(ncFileID, DNWVarID(id), "long_name", &
        "dn values on full (w) levels"))
   call check(nf90_put_att(ncFileID, DNWVarID(id), "units", "dimensionless"))

!
!    float MUB(Time, south_north, west_east) ;
!            MUB:FieldType = 104 ;
!            MUB:MemoryOrder = "XY " ;
!            MUB:stagger = "" ;
   call check(nf90_def_var(ncFileID, name="MUB_d0"//idom, xtype=nf90_real, &
        dimids= (/ weDimID(id), snDimID(id) /), varid=MubVarID(id)) )
   call check(nf90_put_att(ncFileID, MubVarID(id), "long_name", &
        "base state dry air mass in column"))
   call check(nf90_put_att(ncFileID, MubVarID(id), "units", "Pa"))

! Longitudes
!      float XLONG(Time, south_north, west_east) ;
!         XLONG:FieldType = 104 ;
!         XLONG:MemoryOrder = "XY " ;
!         XLONG:stagger = "" ;
   call check(nf90_def_var(ncFileID, name="XLON_d0"//idom, xtype=nf90_real, &
        dimids= (/ weDimID(id), snDimID(id) /), varid=LonVarID(id)) )
   call check(nf90_put_att(ncFileID, LonVarID(id), "long_name", "longitude"))
   call check(nf90_put_att(ncFileID, LonVarID(id), "units", "degrees_east"))
   call check(nf90_put_att(ncFileID, LonVarID(id), "valid_range", &
        (/ -180.0_r8, 180.0_r8 /)))
   call check(nf90_put_att(ncFileID, LonVarID(id), "description", &
        "LONGITUDE, WEST IS NEGATIVE"))

! Latitudes
!      float XLAT(Time, south_north, west_east) ;
!         XLAT:FieldType = 104 ;
!         XLAT:MemoryOrder = "XY " ;
!         XLAT:stagger = "" ;
   call check(nf90_def_var(ncFileID, name="XLAT_d0"//idom, xtype=nf90_real, &
        dimids=(/ weDimID(id), snDimID(id) /), varid=LatVarID(id)) ) 
   call check(nf90_put_att(ncFileID, LatVarID(id), "long_name", "latitude"))
   call check(nf90_put_att(ncFileID, LatVarID(id), "units", "degrees_north"))
   call check(nf90_put_att(ncFileID, LatVarID(id), "valid_range", &
        (/ -90.0_r8, 90.0_r8 /)))
   call check(nf90_put_att(ncFileID, LatVarID(id), "description", &
        "LATITUDE, SOUTH IS NEGATIVE"))

! grid levels
   call check(nf90_def_var(ncFileID, name="level_d0"//idom, xtype=nf90_short, &
        dimids=btDimID(id), varid=ilevVarID(id)) )
   call check(nf90_put_att(ncFileID, ilevVarID(id), "long_name", &
        "placeholder for level"))
   call check(nf90_put_att(ncFileID, ilevVarID(id), "units", &
        "at this point, indexical"))

! Land Mask
!    float XLAND(Time, south_north, west_east) ;
!            XLAND:FieldType = 104 ;
!            XLAND:MemoryOrder = "XY " ;
!            XLAND:units = "NA" ;
!            XLAND:stagger = "" ;
   call check(nf90_def_var(ncFileID, name="XLAND_d0"//idom, xtype=nf90_short, &
        dimids= (/ weDimID(id), snDimID(id) /), varid=XlandVarID(id)) )
   call check(nf90_put_att(ncFileID, XlandVarID(id), "long_name", "land mask"))
   call check(nf90_put_att(ncFileID, XlandVarID(id), "units", "NA"))
   call check(nf90_put_att(ncFileID, XlandVarID(id), "valid_range", (/ 1, 2 /)))
   call check(nf90_put_att(ncFileID, XlandVarID(id), "description", &
        "1 = LAND, 2 = WATER"))

! Map Scale Factor on m-grid
!    float MAPFAC_M(Time, south_north, west_east) ;
!            MAPFAC_M:FieldType = 104 ;
!            MAPFAC_M:MemoryOrder = "XY " ;
!            MAPFAC_M:stagger = "" ;
   call check(nf90_def_var(ncFileID, name="MAPFAC_M_d0"//idom, xtype=nf90_real, &
        dimids= (/ weDimID(id), snDimID(id) /), varid=MapFacMVarID(id)) )
   call check(nf90_put_att(ncFileID, MapFacMVarID(id), "long_name", &
        "Map scale factor on mass grid"))
   call check(nf90_put_att(ncFileID, MapFacMVarID(id), "units", "dimensionless"))

! Map Scale Factor on u-grid
!    float MAPFAC_U(Time, south_north, west_east_stag) ;
!            MAPFAC_U:FieldType = 104 ;
!            MAPFAC_U:MemoryOrder = "XY " ;
!            MAPFAC_U:stagger = "X" ;
   call check(nf90_def_var(ncFileID, name="MAPFAC_U_d0"//idom, xtype=nf90_real, &
        dimids= (/ weStagDimID(id), snDimID(id) /), varid=MapFacUVarID(id)) )
   call check(nf90_put_att(ncFileID, MapFacUVarID(id), "long_name", &
        "Map scale factor on u-grid"))
   call check(nf90_put_att(ncFileID, MapFacUVarID(id), "units", "dimensionless"))

! Map Scale Factor on v-grid
!    float MAPFAC_V(Time, south_north_stag, west_east) ;
!            MAPFAC_V:FieldType = 104 ;
!            MAPFAC_V:MemoryOrder = "XY " ;
!            MAPFAC_V:stagger = "Y" ;
   call check(nf90_def_var(ncFileID, name="MAPFAC_V_d0"//idom, xtype=nf90_real, &
        dimids= (/ weDimID(id), snStagDimID(id) /), varid=MapFacVVarID(id)) )
   call check(nf90_put_att(ncFileID, MapFacVVarID(id), "long_name", &
        "Map scale factor on v-grid"))
   call check(nf90_put_att(ncFileID, MapFacVVarID(id), "units", "dimensionless"))

! PHB
!    float PHB(Time, bottom_top_stag, south_north, west_east) ;
!            PHB:FieldType = 104 ;
!            PHB:MemoryOrder = "XYZ" ;
!            PHB:stagger = "Z" ;
   call check(nf90_def_var(ncFileID, name="PHB_d0"//idom, xtype=nf90_real, &
        dimids= (/ weDimID(id), snDimID(id), btStagDimID(id) /), varid=phbVarId(id)) )
   call check(nf90_put_att(ncFileID, phbVarId(id), "long_name", &
        "base-state geopotential"))
   call check(nf90_put_att(ncFileID, phbVarId(id), "units", "m^2/s^2"))
   call check(nf90_put_att(ncFileID, phbVarId(id), "units_long_name", "m{2} s{-2}"))

   call check(nf90_def_var(ncFileID, name="HGT_d0"//idom, xtype=nf90_real, &
        dimids= (/ weDimID(id), snDimID(id) /), varid=hgtVarId(id)) )
   call check(nf90_put_att(ncFileID, hgtVarId(id), "long_name", "Terrain Height"))
   call check(nf90_put_att(ncFileID, hgtVarId(id), "units", "m"))
   call check(nf90_put_att(ncFileID, hgtVarId(id), "units_long_name", "meters"))

enddo

if ( output_state_vector ) then

   !-----------------------------------------------------------------
   ! Create attributes for the state vector 
   !-----------------------------------------------------------------

   ! Define the state vector coordinate variable

   call check(nf90_def_var(ncid=ncFileID,name="StateVariable", xtype=nf90_int, &
              dimids=StateVarDimID, varid=StateVarVarID))

   call check(nf90_put_att(ncFileID, StateVarVarID, "long_name", &
        "State Variable ID"))
   call check(nf90_put_att(ncFileID, StateVarVarID, "units", &
        "indexical") )
   call check(nf90_put_att(ncFileID, StateVarVarID, "valid_range", &
        (/ 1, wrf%model_size /)))

   ! Define the actual state vector

   call check(nf90_def_var(ncid=ncFileID, name="state", xtype=nf90_real, &
              dimids = (/ StateVarDimID, MemberDimID, unlimitedDimID /), &
              varid=StateVarID))
   call check(nf90_put_att(ncFileID, StateVarID, "long_name", &
        "model state or fcopy"))
   call check(nf90_put_att(ncFileID, StateVarId, "U_units","m/s"))
   call check(nf90_put_att(ncFileID, StateVarId, "V_units","m/s"))
   call check(nf90_put_att(ncFileID, StateVarId, "W_units","m/s"))
   call check(nf90_put_att(ncFileID, StateVarId, "GZ_units","m2/s2"))
   call check(nf90_put_att(ncFileID, StateVarId, "T_units","K"))
   call check(nf90_put_att(ncFileID, StateVarId, "MU_units","Pa"))
   call check(nf90_put_att(ncFileID, StateVarId, "TSLB_units","K"))
   call check(nf90_put_att(ncFileID, StateVarId, "TSK_units","K"))
   if( wrf%dom(num_domains)%n_moist >= 1) then
      call check(nf90_put_att(ncFileID, StateVarId, "QV_units","kg/kg"))
   endif
   if( wrf%dom(num_domains)%n_moist >= 2) then
      call check(nf90_put_att(ncFileID, StateVarId, "QC_units","kg/kg"))
   endif
   if( wrf%dom(num_domains)%n_moist >= 3) then
      call check(nf90_put_att(ncFileID, StateVarId, "QR_units","kg/kg"))
   endif
   if( wrf%dom(num_domains)%n_moist >= 4) then
      call check(nf90_put_att(ncFileID, StateVarId, "QI_units","kg/kg"))
   endif
   if( wrf%dom(num_domains)%n_moist >= 5) then
      call check(nf90_put_att(ncFileID, StateVarId, "QS_units","kg/kg"))
   endif
   if( wrf%dom(num_domains)%n_moist == 6) then
      call check(nf90_put_att(ncFileID, StateVarId, "QG_units","kg/kg"))
   endif
   if(wrf%dom(num_domains)%surf_obs ) then
      call check(nf90_put_att(ncFileID, StateVarId, "U10_units","m/s"))
      call check(nf90_put_att(ncFileID, StateVarId, "V10_units","m/s"))
      call check(nf90_put_att(ncFileID, StateVarId, "T2_units","K"))
      call check(nf90_put_att(ncFileID, StateVarId, "Q2_units","kg/kg"))
      call check(nf90_put_att(ncFileID, StateVarId, "PS_units","Pa"))
   endif

   ! Leave define mode so we can actually fill the variables.

   call check(nf90_enddef(ncfileID))

   call check(nf90_put_var(ncFileID, StateVarVarID, &
        (/ (i,i=1,wrf%model_size) /) ))

else

do id=1,num_domains
   write( idom , '(I1)') id

   !----------------------------------------------------------------------------
   ! Create the (empty) Prognostic Variables and their attributes
   !----------------------------------------------------------------------------

   !      float U(Time, bottom_top, south_north, west_east_stag) ;
   !         U:FieldType = 104 ;
   !         U:MemoryOrder = "XYZ" ;
   !         U:stagger = "X" ;
   call check(nf90_def_var(ncid=ncFileID, name="U_d0"//idom, xtype=nf90_real, &
         dimids = (/ weStagDimID(id), snDimId(id), btDimID(id), MemberDimID, &
         unlimitedDimID /), varid  = var_id))
   call check(nf90_put_att(ncFileID, var_id, "long_name", "x-wind component"))
   call check(nf90_put_att(ncFileID, var_id, "units", "m/s"))
   call check(nf90_put_att(ncFileID, var_id, "units_long_name", "m s{-1}"))


   !      float V(Time, bottom_top, south_north_stag, west_east) ;
   !         V:FieldType = 104 ;
   !         V:MemoryOrder = "XYZ" ;
   !         V:stagger = "Y" ;
   call check(nf90_def_var(ncid=ncFileID, name="V_d0"//idom, xtype=nf90_real, &
         dimids = (/ weDimID(id), snStagDimID(id), btDimID(id), MemberDimID, &
         unlimitedDimID /), varid  = var_id))
   call check(nf90_put_att(ncFileID, var_id, "long_name", "y-wind component"))
   call check(nf90_put_att(ncFileID, var_id, "units", "m/s"))
   call check(nf90_put_att(ncFileID, var_id, "units_long_name", "m s{-1}"))


   !      float W(Time, bottom_top_stag, south_north, west_east) ;
   !         W:FieldType = 104 ;
   !         W:MemoryOrder = "XYZ" ;
   !         W:stagger = "Z" ;
   call check(nf90_def_var(ncid=ncFileID, name="W_d0"//idom, xtype=nf90_real, &
         dimids = (/ weDimID(id), snDimID(id), btStagDimID(id), MemberDimID, &
         unlimitedDimID /), varid  = var_id))
   call check(nf90_put_att(ncFileID, var_id, "long_name", "z-wind component"))
   call check(nf90_put_att(ncFileID, var_id, "units", "m/s"))
   call check(nf90_put_att(ncFileID, var_id, "units_long_name", "m s{-1}"))


   !      float PH(Time, bottom_top_stag, south_north, west_east) ;
   !         PH:FieldType = 104 ;
   !         PH:MemoryOrder = "XYZ" ;
   !         PH:stagger = "Z" ;
   call check(nf90_def_var(ncid=ncFileID, name="PH_d0"//idom, xtype=nf90_real, &
         dimids = (/ weDimID(id), snDimID(id), btStagDimID(id), MemberDimID, &
         unlimitedDimID /), varid  = var_id))
   call check(nf90_put_att(ncFileID, var_id, "long_name", &
        "perturbation geopotential"))
   call check(nf90_put_att(ncFileID, var_id, "units", "m^2/s^2"))
   call check(nf90_put_att(ncFileID, var_id, "units_long_name", "m{2} s{-2}"))


   !      float T(Time, bottom_top, south_north, west_east) ;
   !         T:FieldType = 104 ;
   !         T:MemoryOrder = "XYZ" ;
   !         T:units = "K" ;
   !         T:stagger = "" ;
   call check(nf90_def_var(ncid=ncFileID, name="T_d0"//idom, xtype=nf90_real, &
         dimids = (/ weDimID(id), snDimID(id), btDimID(id), MemberDimID, &
         unlimitedDimID /), varid  = var_id))
   call check(nf90_put_att(ncFileID, var_id, "long_name", "temperature"))
   call check(nf90_put_att(ncFileID, var_id, "units", "K"))
   call check(nf90_put_att(ncFileId, var_id, "description", &
        "perturbation potential temperature (theta-t0)"))


   !      float MU(Time, south_north, west_east) ;
   !         MU:FieldType = 104 ;
   !         MU:MemoryOrder = "XY " ;
   !         MU:description = "perturbation dry air mass in column" ;
   !         MU:units = "pascals" ;
   !         MU:stagger = "" ;
   call check(nf90_def_var(ncid=ncFileID, name="MU_d0"//idom, xtype=nf90_real, &
         dimids = (/ weDimID(id), snDimID(id), MemberDimID, &
         unlimitedDimID /), varid  = var_id))
   call check(nf90_put_att(ncFileID, var_id, "long_name", "mu field"))
   call check(nf90_put_att(ncFileID, var_id, "units", "pascals"))
   call check(nf90_put_att(ncFileId, var_id, "description", &
        "perturbation dry air mass in column"))


   !      float TSLB(Time, soil_layers_stag, south_north, west_east) ;
   !         TSLB:FieldType = 104 ;
   !         TSLB:MemoryOrder = "XYZ" ;
   !         TSLB:description = "SOIL TEMPERATURE" ;
   !         TSLB:units = "K" ;
   !         TSLB:stagger = "Z" ;
   call check(nf90_def_var(ncid=ncFileID, name="TSLB_d0"//idom, xtype=nf90_real, &
         dimids = (/ weDimID(id), snDimID(id), slSDimID(id), MemberDimID, &
         unlimitedDimID /), varid  = var_id))
   call check(nf90_put_att(ncFileID, var_id, "long_name", "soil temperature"))
   call check(nf90_put_att(ncFileID, var_id, "units", "K"))
   call check(nf90_put_att(ncFileId, var_id, "description", &
        "SOIL TEMPERATURE"))


   !      float TSK(Time, south_north, west_east) ;
   !         TSK:FieldType = 104 ;
   !         TSK:MemoryOrder = "XY " ;
   !         TSK:description = "SURFACE SKIN TEMPERATURE" ;
   !         TSK:units = "K" ;
   !         TSK:stagger = "" ;
   call check(nf90_def_var(ncid=ncFileID, name="TSK_d0"//idom, xtype=nf90_real, &
         dimids = (/ weDimID(id), snDimID(id), MemberDimID, &
         unlimitedDimID /), varid  = var_id))
   call check(nf90_put_att(ncFileID, var_id, "long_name", "tsk field"))
   call check(nf90_put_att(ncFileID, var_id, "units", "K"))
   call check(nf90_put_att(ncFileId, var_id, "description", &
        "SURFACE SKIN TEMPERATURE"))


   !      float QVAPOR(Time, bottom_top, south_north, west_east) ;
   !         QVAPOR:FieldType = 104 ;
   !         QVAPOR:MemoryOrder = "XYZ" ;
   !         QVAPOR:stagger = "" ;
   if( wrf%dom(id)%n_moist >= 1) then
      call check(nf90_def_var(ncid=ncFileID, name="QVAPOR_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), btDimID(id), MemberDimID, &
           unlimitedDimID /), varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "kg/kg"))
      call check(nf90_put_att(ncFileId, var_id, "description", &
           "Water vapor mixing ratio"))
   endif


   !      float QCLOUD(Time, bottom_top, south_north, west_east) ;
   !         QCLOUD:FieldType = 104 ;
   !         QCLOUD:MemoryOrder = "XYZ" ;
   !         QCLOUD:stagger = "" ;
   if( wrf%dom(id)%n_moist >= 2) then
      call check(nf90_def_var(ncid=ncFileID, name="QCLOUD_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), btDimID(id), MemberDimID, &
           unlimitedDimID /), varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "kg/kg"))
      call check(nf90_put_att(ncFileID, var_id, "description", &
           "Cloud water mixing ratio"))
   endif


   !      float QRAIN(Time, bottom_top, south_north, west_east) ;
   !         QRAIN:FieldType = 104 ;
   !         QRAIN:MemoryOrder = "XYZ" ;
   !         QRAIN:stagger = "" ;
   if( wrf%dom(id)%n_moist >= 3) then
      call check(nf90_def_var(ncid=ncFileID, name="QRAIN_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), btDimID(id), MemberDimID, &
           unlimitedDimID /), varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "kg/kg"))
      call check(nf90_put_att(ncFileID, var_id, "description", &
           "Rain water mixing ratio"))
   endif

   if( wrf%dom(id)%n_moist >= 4) then
      call check(nf90_def_var(ncid=ncFileID, name="QICE_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), btDimID(id), MemberDimID, &
           unlimitedDimID /), varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "kg/kg"))
      call check(nf90_put_att(ncFileID, var_id, "description", &
           "Ice mixing ratio"))
   endif

   if( wrf%dom(id)%n_moist >= 5) then
      call check(nf90_def_var(ncid=ncFileID, name="QSNOW_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), btDimID(id), MemberDimID, &
           unlimitedDimID /), varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "kg/kg"))
      call check(nf90_put_att(ncFileID, var_id, &
           "description", "Snow mixing ratio"))
   endif

   if( wrf%dom(id)%n_moist == 6) then
      call check(nf90_def_var(ncid=ncFileID, name="QGRAUP_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), btDimID(id), MemberDimID, &
           unlimitedDimID /), varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "kg/kg"))
      call check(nf90_put_att(ncFileID, var_id, "description", &
           "Graupel mixing ratio"))
   endif

   if(wrf%dom(id)%surf_obs ) then

      call check(nf90_def_var(ncid=ncFileID, name="U10_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), MemberDimID, unlimitedDimID /), &
           varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "m/s"))
      call check(nf90_put_att(ncFileID, var_id, "description", "U at 10 m"))

      call check(nf90_def_var(ncid=ncFileID, name="V10_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), MemberDimID, unlimitedDimID /), &
           varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "m/s"))
      call check(nf90_put_att(ncFileID, var_id, "description", "V at 10 m"))

      call check(nf90_def_var(ncid=ncFileID, name="T2_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), MemberDimID, unlimitedDimID /), &
           varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "K"))
      call check(nf90_put_att(ncFileID, var_id, "description", "TEMP at 2 m"))

      call check(nf90_def_var(ncid=ncFileID, name="Q2_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), MemberDimID, unlimitedDimID /), &
           varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "kg/kg"))
      call check(nf90_put_att(ncFileID, var_id, "description", "QV at 2 m"))

      call check(nf90_def_var(ncid=ncFileID, name="PS_d0"//idom, xtype=nf90_real, &
           dimids = (/ weDimID(id), snDimID(id), MemberDimID, unlimitedDimID /), &
           varid  = var_id))
      call check(nf90_put_att(ncFileID, var_id, "units", "Pa"))
      call check(nf90_put_att(ncFileID, var_id, "description", "Total surface pressure"))

   end if

enddo

endif

!-----------------------------------------------------------------
! Fill the variables we can
!-----------------------------------------------------------------
call check(nf90_enddef(ncfileID))

call check(nf90_put_var(ncFileID,       DXVarID, wrf%dom(1:num_domains)%dx        ))
call check(nf90_put_var(ncFileID,       DYVarID, wrf%dom(1:num_domains)%dy        ))
call check(nf90_put_var(ncFileID, TRUELAT1VarID, wrf%dom(1:num_domains)%proj%truelat1  ))
call check(nf90_put_var(ncFileID, TRUELAT2VarID, wrf%dom(1:num_domains)%proj%truelat2  ))
call check(nf90_put_var(ncFileID,  CEN_LATVarID, wrf%dom(1:num_domains)%cen_lat   ))
call check(nf90_put_var(ncFileID,  CEN_LONVarID, wrf%dom(1:num_domains)%cen_lon   ))
call check(nf90_put_var(ncFileID, MAP_PROJVarID, wrf%dom(1:num_domains)%map_proj  ))

do id=1,num_domains

! defining grid levels
   call check(nf90_put_var(ncFileID,       DNVarID(id), wrf%dom(id)%dn        ))
   call check(nf90_put_var(ncFileID,      ZNUVarID(id), wrf%dom(id)%znu       ))
   call check(nf90_put_var(ncFileID,      DNWVarID(id), wrf%dom(id)%dnw       ))

! defining horizontal
   call check(nf90_put_var(ncFileID,      mubVarID(id), wrf%dom(id)%mub       ))
   call check(nf90_put_var(ncFileID,      LonVarID(id), wrf%dom(id)%longitude ))
   call check(nf90_put_var(ncFileID,      LatVarID(id), wrf%dom(id)%latitude  ))
   call check(nf90_put_var(ncFileID,     ilevVarID(id), (/ (i,i=1,wrf%dom(id)%bt) /) ))
   call check(nf90_put_var(ncFileID,    XlandVarID(id), wrf%dom(id)%land      ))
   call check(nf90_put_var(ncFileID,  MapFacMVarID(id), wrf%dom(id)%mapfac_m  ))
   call check(nf90_put_var(ncFileID,  MapFacUVarID(id), wrf%dom(id)%mapfac_u  ))
   call check(nf90_put_var(ncFileID,  MapFacVVarID(id), wrf%dom(id)%mapfac_v  ))
   call check(nf90_put_var(ncFileID,      phbVarID(id), wrf%dom(id)%phb       ))
   call check(nf90_put_var(ncFileID,      hgtVarID(id), wrf%dom(id)%hgt       ))

enddo

!-----------------------------------------------------------------
! Flush the buffer and leave netCDF file open
!-----------------------------------------------------------------

call check(nf90_sync(ncFileID))

write (*,*)'nc_write_model_atts: netCDF file ',ncFileID,' is synched ...'

contains

  ! Internal subroutine - checks error status after each netcdf, prints
  !                       text message each time an error code is returned.
  subroutine check(istatus)
    integer, intent ( in) :: istatus

    if(istatus /= nf90_noerr) call error_handler(E_ERR, 'nc_write_model_atts', &
       trim(nf90_strerror(istatus)), source, revision, revdate)

  end subroutine check

end function nc_write_model_atts



function nc_write_model_vars( ncFileID, statevec, copyindex, timeindex ) result (ierr)
!-----------------------------------------------------------------
! Writes the model-specific variables to a netCDF file
! TJH 25 June 2003
!
! TJH 29 July 2003 -- for the moment, all errors are fatal, so the
! return code is always '0 == normal', since the fatal errors stop execution.


integer,                intent(in) :: ncFileID      ! netCDF file identifier
real(r8), dimension(:), intent(in) :: statevec
integer,                intent(in) :: copyindex
integer,                intent(in) :: timeindex
integer                            :: ierr          ! return value of function

!-----------------------------------------------------------------

logical, parameter :: debug = .false.  
integer :: nDimensions, nVariables, nAttributes, unlimitedDimID
integer :: StateVarID, VarID, id
integer :: i,j
real(r8), allocatable, dimension(:,:)   :: temp2d
real(r8), allocatable, dimension(:,:,:) :: temp3d
character(len=10) :: varname
character(len=1) :: idom

ierr = 0     ! assume normal termination

!-----------------------------------------------------------------
! make sure ncFileID refers to an open netCDF file, 
! then get all the Variable ID's we need.
!-----------------------------------------------------------------

call check(nf90_Inquire(ncFileID, nDimensions, nVariables, nAttributes, unlimitedDimID))

if ( output_state_vector ) then

   call check(NF90_inq_varid(ncFileID, "state", StateVarID) )
   call check(NF90_put_var(ncFileID, StateVarID, statevec,  &
                start=(/ 1, copyindex, timeindex /)))

else

j = 0

do id=1,num_domains

   write( idom , '(I1)') id

   !----------------------------------------------------------------------------
   ! Fill the variables, the order is CRITICAL  ...   U,V,W,GZ,T,MU,TSLB,TSK,QV,QC,QR,...
   !----------------------------------------------------------------------------

   !----------------------------------------------------------------------------
   varname = 'U_d0'//idom
   !----------------------------------------------------------------------------
   call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
   i       = j + 1
   j       = i + wrf%dom(id)%wes * wrf%dom(id)%sn * wrf%dom(id)%bt - 1 
   if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
              trim(adjustl(varname)),i,j,wrf%dom(id)%wes,wrf%dom(id)%sn,wrf%dom(id)%bt 
   allocate ( temp3d(wrf%dom(id)%wes, wrf%dom(id)%sn, wrf%dom(id)%bt) )
   temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%wes, wrf%dom(id)%sn, wrf%dom(id)%bt /) ) 
   call check(nf90_put_var( ncFileID, VarID, temp3d, &
                            start=(/ 1, 1, 1, copyindex, timeindex /) ))
   deallocate(temp3d)


   !----------------------------------------------------------------------------
   varname = 'V_d0'//idom
   !----------------------------------------------------------------------------
   call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
   i       = j + 1
   j       = i + wrf%dom(id)%we * wrf%dom(id)%sns * wrf%dom(id)%bt - 1
   if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
              trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sns,wrf%dom(id)%bt
   allocate ( temp3d(wrf%dom(id)%we, wrf%dom(id)%sns, wrf%dom(id)%bt) )
   temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sns, wrf%dom(id)%bt /) ) 
   call check(nf90_put_var( ncFileID, VarID, temp3d, &
                            start=(/ 1, 1, 1, copyindex, timeindex /) ))
   deallocate(temp3d)


   !----------------------------------------------------------------------------
   varname = 'W_d0'//idom
   !----------------------------------------------------------------------------
   call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
   i       = j + 1
   j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%bts - 1
   if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
              trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bts
   allocate ( temp3d(wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bts) )
   temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bts /) ) 
   call check(nf90_put_var( ncFileID, VarID, temp3d, &
                            start=(/ 1, 1, 1, copyindex, timeindex /) ))


   !----------------------------------------------------------------------------
   varname = 'PH_d0'//idom       ! AKA "GZ"
   !----------------------------------------------------------------------------
   call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
   i       = j + 1
   j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%bts - 1
   if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
              trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bts
   temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bts /) ) 
   call check(nf90_put_var( ncFileID, VarID, temp3d, &
                            start=(/ 1, 1, 1, copyindex, timeindex /) ))
   deallocate(temp3d)


   !----------------------------------------------------------------------------
   varname = 'T_d0'//idom
   !----------------------------------------------------------------------------
   call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
   i       = j + 1
   j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%bt - 1
   if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
              trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bt
   allocate ( temp3d(wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bt) )
   temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bt /) ) 
   call check(nf90_put_var( ncFileID, VarID, temp3d, &
                            start=(/ 1, 1, 1, copyindex, timeindex /) ))
   deallocate(temp3d)


   !----------------------------------------------------------------------------
   varname = 'MU_d0'//idom
   !----------------------------------------------------------------------------
   call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
   i       = j + 1
   j       = i + wrf%dom(id)%we * wrf%dom(id)%sn - 1
   if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
              trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn
   allocate ( temp2d(wrf%dom(id)%we, wrf%dom(id)%sn) )
   temp2d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn /) ) 
   call check(nf90_put_var( ncFileID, VarID, temp2d, &
                            start=(/ 1, 1, copyindex, timeindex /) ))


   !----------------------------------------------------------------------------
   varname = 'TSLB_d0'//idom
   !----------------------------------------------------------------------------
   call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
   i       = j + 1
   j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%sls - 1
   if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
              trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%sls
   allocate ( temp3d(wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%sls) )
   temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%sls /) ) 
   call check(nf90_put_var( ncFileID, VarID, temp3d, &
                            start=(/ 1, 1, 1, copyindex, timeindex /) ))
   deallocate(temp3d)


   !----------------------------------------------------------------------------
   varname = 'TSK_d0'//idom
   !----------------------------------------------------------------------------
   call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
   i       = j + 1
   j       = i + wrf%dom(id)%we * wrf%dom(id)%sn - 1
   if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
              trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn
   temp2d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn /) ) 
   call check(nf90_put_var( ncFileID, VarID, temp2d, &
                            start=(/ 1, 1, copyindex, timeindex /) ))


   if( wrf%dom(id)%n_moist >= 1) then
      !----------------------------------------------------------------------------
      varname = 'QVAPOR_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%bt - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bt
      allocate ( temp3d(wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bt) )
      temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bt /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp3d, &
           start=(/ 1, 1, 1, copyindex, timeindex /) ))
   endif


   if( wrf%dom(id)%n_moist >= 2) then
      !----------------------------------------------------------------------------
      varname = 'QCLOUD_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%bt - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bt
      temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bt /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp3d, &
           start=(/ 1, 1, 1, copyindex, timeindex /) ))
   endif


   if( wrf%dom(id)%n_moist >= 3) then
      !----------------------------------------------------------------------------
      varname = 'QRAIN_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%bt - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bt
      temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bt /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp3d, &
           start=(/ 1, 1, 1, copyindex, timeindex /) ))
   endif


   if( wrf%dom(id)%n_moist >= 4) then
      !----------------------------------------------------------------------------
      varname = 'QICE_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%bt - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bt
      temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bt /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp3d, &
           start=(/ 1, 1, 1, copyindex, timeindex /) ))
   endif
   if( wrf%dom(id)%n_moist >= 5) then
      !----------------------------------------------------------------------------
      varname = 'QSNOW_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%bt - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bt
      temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bt /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp3d, &
           start=(/ 1, 1, 1, copyindex, timeindex /) ))
   endif
   if( wrf%dom(id)%n_moist == 6) then
      !----------------------------------------------------------------------------
      varname = 'QGRAUP_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn * wrf%dom(id)%bt - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn,wrf%dom(id)%bt
      temp3d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn, wrf%dom(id)%bt /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp3d, &
           start=(/ 1, 1, 1, copyindex, timeindex /) ))
   endif

   deallocate(temp3d)

   if ( wrf%dom(id)%n_moist > 6 ) then
      write(*,'(''wrf%dom(id)%n_moist = '',i3)')wrf%dom(id)%n_moist
      call error_handler(E_ERR,'nc_write_model_vars', &
               'num_moist_vars is too large.', source, revision, revdate)
   endif

   if(wrf%dom(id)%surf_obs ) then

      !----------------------------------------------------------------------------
      varname = 'U10_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn
      temp2d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp2d, &
           start=(/ 1, 1, copyindex, timeindex /) ))

      !----------------------------------------------------------------------------
      varname = 'V10_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn
      temp2d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp2d, &
           start=(/ 1, 1, copyindex, timeindex /) ))

      !----------------------------------------------------------------------------
      varname = 'T2_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn
      temp2d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp2d, &
           start=(/ 1, 1, copyindex, timeindex /) ))

      !----------------------------------------------------------------------------
      varname = 'Q2_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn
      temp2d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp2d, &
           start=(/ 1, 1, copyindex, timeindex /) ))

      !----------------------------------------------------------------------------
      varname = 'PS_d0'//idom
      !----------------------------------------------------------------------------
      call check(NF90_inq_varid(ncFileID, trim(adjustl(varname)), VarID))
      i       = j + 1
      j       = i + wrf%dom(id)%we * wrf%dom(id)%sn - 1
      if (debug) write(*,'(a10,'' = statevec('',i7,'':'',i7,'') with dims '',3(1x,i3))') &
           trim(adjustl(varname)),i,j,wrf%dom(id)%we,wrf%dom(id)%sn
      temp2d  = reshape(statevec(i:j), (/ wrf%dom(id)%we, wrf%dom(id)%sn /) ) 
      call check(nf90_put_var( ncFileID, VarID, temp2d, &
           start=(/ 1, 1, copyindex, timeindex /) ))

   endif

   deallocate(temp2d)

enddo

endif

!-----------------------------------------------------------------
! Flush the buffer and leave netCDF file open
!-----------------------------------------------------------------

write (*,*)'Finished filling variables ...'
call check(nf90_sync(ncFileID))
write (*,*)'netCDF file is synched ...'

contains

  ! Internal subroutine - checks error status after each netcdf, prints
  !                       text message each time an error code is returned.
  subroutine check(istatus)
    integer, intent ( in) :: istatus

    if(istatus /= nf90_noerr) call error_handler(E_ERR, 'nc_write_model_vars', &
         trim(nf90_strerror(istatus)), source, revision, revdate)

  end subroutine check

end function nc_write_model_vars

!-------------------------------

!  public stubs

!**********************************************

subroutine adv_1step(x, Time)

! Does single time-step advance with vector state as
! input and output.

  real(r8), intent(inout) :: x(:)

! Time is needed for more general models like this; need to add in to 
! low-order models
  type(time_type), intent(in) :: Time

end subroutine adv_1step

!**********************************************

subroutine end_model()
end subroutine end_model

!**********************************************

subroutine init_time(i_time)
! For now returns value of Time_init which is set in initialization routines.

  type(time_type), intent(out) :: i_time

!Where should initial time come from here?
! WARNING: CURRENTLY SET TO 0
  i_time = set_time(0, 0)

end subroutine init_time

!**********************************************

subroutine init_conditions(x)
! Reads in restart initial conditions and converts to vector

! Following changed to intent(inout) for ifc compiler;should be like this
  real(r8), intent(inout) :: x(:)

end subroutine init_conditions



!#######################################################################

subroutine toGrid (x, j, dx, dxm)

!  Transfer obs. x to grid j and calculate its
!  distance to grid j and j+1

  real(r8), intent(in)  :: x
  real(r8), intent(out) :: dx, dxm
  integer,  intent(out) :: j

  j = int (x)

  dx = x - real (j)

  dxm= 1.0_r8 - dx

end subroutine toGrid

!#######################################################################

subroutine pres_to_zk(pres, mdl_v, n3, zk)

! Calculate the model level "zk" on half (mass) levels,
! corresponding to pressure "pres".

  integer,  intent(in)  :: n3
  real(r8), intent(in)  :: pres
  real(r8), intent(in)  :: mdl_v(0:n3)
  real(r8), intent(out) :: zk

  integer  :: k

  zk = missing_r8

  if (pres > mdl_v(0) .or. pres < mdl_v(n3)) return

  do k = 0,n3-1
     if(pres <= mdl_v(k) .and. pres >= mdl_v(k+1)) then
        zk = real(k) + (mdl_v(k) - pres)/(mdl_v(k) - mdl_v(k+1))
        exit
     endif
  enddo

end subroutine pres_to_zk

!#######################################################################

subroutine height_to_zk(obs_v, mdl_v, n3, zk)

! Calculate the model level "zk" on half (mass) levels,
! corresponding to height "obs_v".

  real(r8), intent(in)  :: obs_v
  integer,  intent(in)  :: n3
  real(r8), intent(in)  :: mdl_v(0:n3)
  real(r8), intent(out) :: zk

  integer   :: k

  zk = missing_r8

  if (obs_v < mdl_v(0) .or. obs_v > mdl_v(n3)) return

  do k = 0,n3-1
     if(obs_v >= mdl_v(k) .and. obs_v <= mdl_v(k+1)) then
        zk = real(k) + (mdl_v(k) - obs_v)/(mdl_v(k) - mdl_v(k+1))
        exit
     endif
  enddo

end subroutine height_to_zk

!#######################################################

subroutine get_model_pressure_profile(i,j,dx,dy,dxm,dym,n,x,id,v_p)

! Calculate the full model pressure profile on half (mass) levels,
! horizontally interpolated at the observation location.

integer,  intent(in)  :: i,j,n,id
real(r8), intent(in)  :: dx,dy,dxm,dym
real(r8), intent(in)  :: x(:)
real(r8), intent(out) :: v_p(0:n)

integer  :: i1,i2,k
real(r8) :: pres1, pres2, pres3, pres4

if(i >= 1 .and. i < wrf%dom(id)%var_size(1,TYPE_T) .and. &
   j >= 1 .and. j < wrf%dom(id)%var_size(2,TYPE_T)) then

   do k=1,n
      pres1 = model_pressure_t(i  ,j  ,k,id,x)
      pres2 = model_pressure_t(i+1,j  ,k,id,x)
      pres3 = model_pressure_t(i  ,j+1,k,id,x)
      pres4 = model_pressure_t(i+1,j+1,k,id,x)
      v_p(k) = dym*( dxm*pres1 + dx*pres2 ) + dy*( dxm*pres3 + dx*pres4 )
   enddo

   if(wrf%dom(id)%surf_obs ) then

      i1 = get_wrf_index(i,j,1,TYPE_PS,id)
      i2 = get_wrf_index(i,j+1,1,TYPE_PS,id)
      if(x(i1) /= 0.0_r8 .and. x(i1+1) /= 0.0_r8 .and. &
         x(i2) /= 0.0_r8 .and. x(i2+1) /= 0.0_r8) then

         v_p(0) = dym*( dxm*x(i1) + dx*x(i1+1) ) + dy*( dxm*x(i2) + dx*x(i2+1) )

      else

         pres1 = model_pressure_t(i  ,j  ,2,id,x)
         pres2 = model_pressure_t(i+1,j  ,2,id,x)
         pres3 = model_pressure_t(i  ,j+1,2,id,x)
         pres4 = model_pressure_t(i+1,j+1,2,id,x)
         v_p(0) = (3.0_r8*v_p(1) - &
              dym*( dxm*pres1 + dx*pres2 ) - dy*( dxm*pres3 + dx*pres4 ))/2.0_r8

      endif

   else

      pres1 = model_pressure_t(i  ,j  ,2,id,x)
      pres2 = model_pressure_t(i+1,j  ,2,id,x)
      pres3 = model_pressure_t(i  ,j+1,2,id,x)
      pres4 = model_pressure_t(i+1,j+1,2,id,x)
      v_p(0) = (3.0_r8*v_p(1) - &
           dym*( dxm*pres1 + dx*pres2 ) - dy*( dxm*pres3 + dx*pres4 ))/2.0_r8

   endif

else

   v_p(:) = missing_r8

endif

end subroutine get_model_pressure_profile

!#######################################################

function model_pressure(i,j,k,id,var_type,x)

! Calculate the pressure at grid point (i,j,k), domain id.
! The grid is defined according to var_type.

integer,  intent(in)  :: i,j,k,id,var_type
real(r8), intent(in)  :: x(:)
real(r8)              :: model_pressure

integer  :: ips, imu
real(r8) :: pres1, pres2

model_pressure = missing_r8

if( (var_type == type_w) .or. (var_type == type_gz) ) then

   if( k == 1 ) then

      pres1 = model_pressure_t(i,j,k,id,x)
      pres2 = model_pressure_t(i,j,k+1,id,x)
      model_pressure = (3.0_r8*pres1 - pres2)/2.0_r8

   elseif( k == wrf%dom(id)%var_size(3,TYPE_W) ) then

      pres1 = model_pressure_t(i,j,k-1,id,x)
      pres2 = model_pressure_t(i,j,k-2,id,x)
      model_pressure = (3.0_r8*pres1 - pres2)/2.0_r8

   else

      pres1 = model_pressure_t(i,j,k,id,x)
      pres2 = model_pressure_t(i,j,k-1,id,x)
      model_pressure = (pres1 + pres2)/2.0_r8

   endif

elseif( var_type == type_u ) then

   if( i == wrf%dom(id)%var_size(1,TYPE_U) ) then

      pres1 = model_pressure_t(i-1,j,k,id,x)
      pres2 = model_pressure_t(i-2,j,k,id,x)
      model_pressure = (3.0_r8*pres1 - pres2)/2.0_r8

   elseif( i == 1 ) then

      pres1 = model_pressure_t(i,j,k,id,x)
      pres2 = model_pressure_t(i+1,j,k,id,x)
      model_pressure = (3.0_r8*pres1 - pres2)/2.0_r8

   else

      pres1 = model_pressure_t(i,j,k,id,x)
      pres2 = model_pressure_t(i-1,j,k,id,x)
      model_pressure = (pres1 + pres2)/2.0_r8

   endif

elseif( var_type == type_v ) then

   if( j == wrf%dom(id)%var_size(2,TYPE_V) ) then

      pres1 = model_pressure_t(i,j-1,k,id,x)
      pres2 = model_pressure_t(i,j-2,k,id,x)
      model_pressure = (3.0_r8*pres1 - pres2)/2.0_r8

   elseif( j == 1 ) then

      pres1 = model_pressure_t(i,j,k,id,x)
      pres2 = model_pressure_t(i,j+1,k,id,x)
      model_pressure = (3.0_r8*pres1 - pres2)/2.0_r8

   else

      pres1 = model_pressure_t(i,j,k,id,x)
      pres2 = model_pressure_t(i,j-1,k,id,x)
      model_pressure = (pres1 + pres2)/2.0_r8

   endif

elseif( var_type == type_mu  .or. var_type == type_tslb .or. &
        var_type == type_ps  .or. var_type == type_u10 .or. &
        var_type == type_v10 .or. var_type == type_t2 .or. &
        var_type == type_q2  .or. var_type == type_tsk) then

   if(wrf%dom(id)%surf_obs ) then

      ips = get_wrf_index(i,j,1,TYPE_PS,id)
      model_pressure = x(ips)

   else

      imu = get_wrf_index(i,j,1,TYPE_MU,id)
      model_pressure = wrf%dom(id)%mub(i,j)+x(imu)

   endif

else

   model_pressure = model_pressure_t(i,j,k,id,x)

endif

end function model_pressure

!#######################################################

function model_pressure_t(i,j,k,id,x)

! Calculate total pressure on mass point (half (mass) levels, T-point).

integer,  intent(in)  :: i,j,k,id
real(r8), intent(in)  :: x(:)
real(r8)              :: model_pressure_t

integer  :: iqv,it
real(r8) :: cpovcv,qvf1,rho

model_pressure_t = missing_r8

! Adapted the code from WRF module_big_step_utilities_em.F ----
!         subroutine calc_p_rho_phi      Y.-R. Guo (10/20/2004)

! Simplification: alb*mub = (phb(i,j,k+1) - phb(i,j,k))/dnw(k)

cpovcv = cp / (cp - gas_constant)

iqv = get_wrf_index(i,j,k,TYPE_QV,id)
it  = get_wrf_index(i,j,k,TYPE_T,id)

qvf1 = 1.0_r8 + x(iqv) / rd_over_rv

rho = model_rho_t(i,j,k,id,x)

! .. total pressure:
model_pressure_t = ps0 * ( (gas_constant*(ts0+x(it))*qvf1) / &
     (ps0/rho) )**cpovcv

end function model_pressure_t

!#######################################################

function model_rho_t(i,j,k,id,x)

! Calculate the total density on mass point (half (mass) levels, T-point).

integer,  intent(in)  :: i,j,k,id
real(r8), intent(in)  :: x(:)
real(r8)              :: model_rho_t

integer  :: imu,iph,iphp1
real(r8) :: ph_e

model_rho_t = missing_r8

! Adapted the code from WRF module_big_step_utilities_em.F ----
!         subroutine calc_p_rho_phi      Y.-R. Guo (10/20/2004)

! Simplification: alb*mub = (phb(i,j,k+1) - phb(i,j,k))/dnw(k)

imu = get_wrf_index(i,j,1,TYPE_MU,id)
iph = get_wrf_index(i,j,k,TYPE_GZ,id)
iphp1 = get_wrf_index(i,j,k+1,TYPE_GZ,id)

ph_e = ( (x(iphp1) + wrf%dom(id)%phb(i,j,k+1)) - (x(iph) + wrf%dom(id)%phb(i,j,k)) ) &
     /wrf%dom(id)%dnw(k)

! now calculate rho = - mu / dphi/deta

model_rho_t = - (wrf%dom(id)%mub(i,j)+x(imu)) / ph_e

end function model_rho_t

!#######################################################

subroutine get_model_height_profile(i,j,dx,dy,dxm,dym,n,x,id,v_h)

! Calculate the model height profile on half (mass) levels,
! horizontally interpolated at the observation location.

integer,  intent(in)  :: i,j,n,id
real(r8), intent(in)  :: dx,dy,dxm,dym
real(r8), intent(in)  :: x(:)
real(r8), intent(out) :: v_h(0:n)

real(r8)  :: fll(n+1)
integer   :: i1,i2,k

if(i >= 1 .and. i < wrf%dom(id)%var_size(1,TYPE_GZ) .and. &
   j >= 1 .and. j < wrf%dom(id)%var_size(2,TYPE_GZ)) then

   do k = 1, wrf%dom(id)%var_size(3,TYPE_GZ)
      i1 = get_wrf_index(i,j,k,TYPE_GZ,id)
      i2 = get_wrf_index(i,j+1,k,TYPE_GZ,id)
      fll(k) = (dym*( dxm*(wrf%dom(id)%phb(i,j,k)+x(i1)) + &
           dx*(wrf%dom(id)%phb(i+1,j,k)+x(i1+1))) + &
           dy*(dxm*(wrf%dom(id)%phb(i,j+1,k)+x(i2)) + &
           dx*(wrf%dom(id)%phb(i+1,j+1,k)+x(i2+1)) ))/gravity
   end do

   do k=1,n
      v_h(k) = 0.5_r8*(fll(k) + fll(k+1) )
   end do

   v_h(0) = dym*( dxm*wrf%dom(id)%hgt(i,j) + &
                   dx*wrf%dom(id)%hgt(i+1,j) ) + &
             dy*( dxm*wrf%dom(id)%hgt(i,j+1) + &
                   dx*wrf%dom(id)%hgt(i+1,j+1) )

else

   print*,'Not able the get height_profile'
   print*,i,j,dx,dy,dxm,dym,n,id,wrf%dom(id)%var_size(1,TYPE_GZ), &
        wrf%dom(id)%var_size(2,TYPE_GZ)

   v_h(:) =  missing_r8

endif

end subroutine get_model_height_profile

!#######################################################

function model_height(i,j,k,id,var_type,x)

integer,  intent(in)  :: i,j,k,id,var_type
real(r8), intent(in)  :: x(:)
real(r8)              :: model_height

integer   :: i1, i2, i3, i4

model_height = missing_r8

if( (var_type == type_w) .or. (var_type == type_gz) ) then

   i1 = get_wrf_index(i,j,k,TYPE_GZ,id)
   model_height = (wrf%dom(id)%phb(i,j,k)+x(i1))/gravity

elseif( var_type == type_u ) then

   if( i == wrf%dom(id)%var_size(1,TYPE_U) ) then

      i1 = get_wrf_index(i-1,j,k  ,TYPE_GZ,id)
      i2 = get_wrf_index(i-1,j,k+1,TYPE_GZ,id)

      model_height = ( 3.0_r8*(wrf%dom(id)%phb(i-1,j,k  )+x(i1)) &
                      +3.0_r8*(wrf%dom(id)%phb(i-1,j,k+1)+x(i2)) &
                             -(wrf%dom(id)%phb(i-2,j,k  )+x(i1-1)) &
                             -(wrf%dom(id)%phb(i-2,j,k+1)+x(i2-1)) )/(4.0_r8*gravity)

   elseif( i == 1 ) then

      i1 = get_wrf_index(i,j,k  ,TYPE_GZ,id)
      i2 = get_wrf_index(i,j,k+1,TYPE_GZ,id)

      model_height = ( 3.0_r8*(wrf%dom(id)%phb(i  ,j,k  )+x(i1)) &
                      +3.0_r8*(wrf%dom(id)%phb(i  ,j,k+1)+x(i2)) &
                             -(wrf%dom(id)%phb(i+1,j,k  )+x(i1+1)) &
                             -(wrf%dom(id)%phb(i+1,j,k+1)+x(i2+1)) )/(4.0_r8*gravity)

   else

      i1 = get_wrf_index(i,j,k  ,TYPE_GZ,id)
      i2 = get_wrf_index(i,j,k+1,TYPE_GZ,id)

      model_height = ( (wrf%dom(id)%phb(i  ,j,k  )+x(i1)) &
                      +(wrf%dom(id)%phb(i  ,j,k+1)+x(i2)) &
                      +(wrf%dom(id)%phb(i-1,j,k  )+x(i1-1)) &
                      +(wrf%dom(id)%phb(i-1,j,k+1)+x(i2-1)) )/(4.0_r8*gravity)

   endif

elseif( var_type == type_v ) then

   if( j == wrf%dom(id)%var_size(2,TYPE_V) ) then

      i1 = get_wrf_index(i,j-1,k  ,TYPE_GZ,id)
      i2 = get_wrf_index(i,j-1,k+1,TYPE_GZ,id)
      i3 = get_wrf_index(i,j-2,k  ,TYPE_GZ,id)
      i4 = get_wrf_index(i,j-2,k+1,TYPE_GZ,id)

      model_height = ( 3.0_r8*(wrf%dom(id)%phb(i,j-1,k  )+x(i1)) &
                      +3.0_r8*(wrf%dom(id)%phb(i,j-1,k+1)+x(i2)) &
                             -(wrf%dom(id)%phb(i,j-2,k  )+x(i3)) &
                             -(wrf%dom(id)%phb(i,j-2,k+1)+x(i4)) )/(4.0_r8*gravity)

   elseif( j == 1 ) then

      i1 = get_wrf_index(i,j  ,k  ,TYPE_GZ,id)
      i2 = get_wrf_index(i,j  ,k+1,TYPE_GZ,id)
      i3 = get_wrf_index(i,j+1,k  ,TYPE_GZ,id)
      i4 = get_wrf_index(i,j+1,k+1,TYPE_GZ,id)

      model_height = ( 3.0_r8*(wrf%dom(id)%phb(i,j  ,k  )+x(i1)) &
                      +3.0_r8*(wrf%dom(id)%phb(i,j  ,k+1)+x(i2)) &
                             -(wrf%dom(id)%phb(i,j+1,k  )+x(i3)) &
                             -(wrf%dom(id)%phb(i,j+1,k+1)+x(i4)) )/(4.0_r8*gravity)

   else

      i1 = get_wrf_index(i,j  ,k  ,TYPE_GZ,id)
      i2 = get_wrf_index(i,j  ,k+1,TYPE_GZ,id)
      i3 = get_wrf_index(i,j-1,k  ,TYPE_GZ,id)
      i4 = get_wrf_index(i,j-1,k+1,TYPE_GZ,id)

      model_height = ( (wrf%dom(id)%phb(i,j  ,k  )+x(i1)) &
                      +(wrf%dom(id)%phb(i,j  ,k+1)+x(i2)) &
                      +(wrf%dom(id)%phb(i,j-1,k  )+x(i3)) &
                      +(wrf%dom(id)%phb(i,j-1,k+1)+x(i4)) )/(4.0_r8*gravity)

   endif

elseif( var_type == type_mu .or. var_type == type_ps .or. &
        var_type == type_tsk) then

   model_height = wrf%dom(id)%hgt(i,j)

elseif( var_type == type_tslb) then

   model_height = wrf%dom(id)%hgt(i,j) - wrf%dom(id)%zs(k)

elseif( var_type == type_u10 .or. var_type == type_v10 ) then

   model_height = wrf%dom(id)%hgt(i,j) + 10.0_r8

elseif( var_type == type_t2 .or. var_type == type_q2 ) then

   model_height = wrf%dom(id)%hgt(i,j) + 2.0_r8

else

   i1 = get_wrf_index(i,j,k  ,TYPE_GZ,id)
   i2 = get_wrf_index(i,j,k+1,TYPE_GZ,id)

   model_height = ( (wrf%dom(id)%phb(i,j,k  )+x(i1)) &
                   +(wrf%dom(id)%phb(i,j,k+1)+x(i2)) )/(2.0_r8*gravity)

endif

end function model_height



subroutine pert_model_state(state, pert_state, interf_provided)
!----------------------------------------------------------------------
! subroutine pert_model_state(state, pert_state, interf_provided)
!
! Perturbs a model state for generating initial ensembles
! Returning interf_provided means go ahead and do this with uniform
! small independent perturbations.

real(r8), intent(in)  :: state(:)
real(r8), intent(out) :: pert_state(:)
logical,  intent(out) :: interf_provided

interf_provided = .false.

end subroutine pert_model_state

!#######################################################

subroutine read_dt_from_wrf_nml(dt)

real(r8), intent(out) :: dt

integer :: time_step
integer :: io, ierr, iunit

namelist /domains/ time_step

iunit = open_file('namelist.input', action = 'read')
read(iunit, nml = domains, iostat = io )
ierr = check_nml_error(io, 'domains')
call close_file(iunit)

! Record the namelist values used for the run ...
call error_handler(E_MSG,'read_dt_from_wrf_nml','domains namelist values are',' ',' ',' ')
write(logfileunit, nml=domains)
write(     *     , nml=domains)

dt = time_step

end subroutine read_dt_from_wrf_nml

end module model_mod
